/*
 * receive code
 * Copyright (C) 1996, 1997  John Ioannidis.
 * Copyright (C) 1998, 1999  Richard Guy Briggs.
 *
 * Modified by secunet AG.
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */

#include <linux/config.h>
#include <linux/version.h>

#define __NO_VERSION__
#include <linux/module.h>

#include <linux/kernel.h> /* printk() */
#include <linux/slab.h> /* kmalloc() */
#include <linux/errno.h>  /* error codes */
#include <linux/types.h>  /* size_t */
#include <linux/interrupt.h> /* mark_bh */

#include <linux/netdevice.h>   /* struct device, and other headers */
#include <linux/etherdevice.h> /* eth_type_trans */
#include <linux/ip.h>          /* struct iphdr */
#include <linux/skbuff.h>
#include <freeswan.h>
#include <linux/spinlock.h> /* *lock* */
#include <asm/uaccess.h>
#include <linux/in6.h>
/* #define proto_priv cb */
#include <asm/checksum.h>
#include <net/ip.h>

#include "radij.h"
#include "ipsec_encap.h"
#include "ipsec_radij.h"
#include "ipsec_netlink.h"
#include "ipsec_xform.h"
#include "ipsec_tunnel.h"
#include "ipsec_rcv.h"
#include "ipsec_ah.h"
#include "ipsec_esp.h"
#include "ipsec_cb.h"

#include <pfkeyv2.h>
#include <pfkey.h>

#ifdef CONFIG_IPSECTHREAD
#  include "ipsec_threads/hsips.h"
#endif

#ifdef CONFIG_IPSEC_PKT_SEQ
#  include "ipsec_threads/seq.h"
#endif

#include <linux/cryptdev.h>

int sysctl_ipsec_inbound_policy_check = 0;

#ifdef CONFIG_DEBUG_CRYPT
extern void dmp(char *s, caddr_t bb, int len);
#endif

#ifdef CONFIG_IPSEC_DEBUG
int debug_ah = 0;
int debug_esp = 0;
int debug_rcv = 0;
#endif /* CONFIG_IPSEC_DEBUG */

__u32 zeroes[AH_AMAX];

#include "pepp_header.h"

int s_to_i(char *p)
#if LEN_PEPP_LEN == 8
{ int ret=0;
  int i,c=0;
  while(++c<8) {
    switch(p[c]) {
     case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
	     i=10+p[c]-'a';
	     break;
     default:
	     i=p[c]-'0';
    }
    ret *=16;
    ret +=i;
  }
  return ret;
}
#elif LEN_PEPP_LEN == 2
{ unsigned char *u=(unsigned char *)p;
  return (u[0]+u[1]*256);
}
#else
#error LEN_PEPP_LEN must be 8 or 2 
#endif


/*
 * Check-replay-window routine, adapted from the original 
 * by J. Hughes, from draft-ietf-ipsec-esp-des-md5-03.txt
 *
 *  This is a routine that implements a 64 packet window. This is intend-
 *  ed on being an implementation sample.
 */

static int
ipsec_checkreplaywindow(struct tdb*tdbp, __u32 seq)
{
	int retval;
	__u32 diff;

	spin_lock(&tdbp->lock);
	
	if (tdbp->tdb_replaywin == 0) {	/* replay shut off */
		retval = 1;
		goto unlock_out_check;
	}

	if (seq == 0) {
		retval = 0;		/* first == 0 or wrapped */
		KLIPS_PRINT(debug_rcv & DB_RX_REPLAY,
			"ipsec_rcv: checkreplaywindow: seq == 0 (seq=%d)\n",
			seq);
		goto unlock_out_check;
	}

	/* new larger sequence number */
	if (seq > tdbp->tdb_replaywin_lastseq) {
		retval = 1;		/* larger is good */
		goto unlock_out_check;
	}
	diff = tdbp->tdb_replaywin_lastseq - seq;

	/* too old or wrapped */ /* if wrapped, kill off SA? */
	if (diff >= tdbp->tdb_replaywin) {
		retval = 0;
		KLIPS_PRINT(debug_rcv & DB_RX_REPLAY,
			"ipsec_rcv: checkreplaywindow: diff >= tdbp->tdb_replaywin\n",
			"(seq=%d, tdb_replaywin_lastseq=%d, diff=%d, tdb_replaywin=%d)\n",
			seq, tdbp->tdb_replaywin_lastseq, diff, tdbp->tdb_replaywin);
		goto unlock_out_check;
	}
	/* this packet already seen */
	if (tdbp->tdb_replaywin_bitmap & (1LL << diff)) {
		retval = 0;
		KLIPS_PRINT(debug_rcv & DB_RX_REPLAY,
			"ipsec_rcv: checkreplaywindow: tdbp->tdb_replaywin_bitmap & (1LL << diff)\n",
			"(seq=%d, tdb_replaywin_lastseq=%d, diff=%d, tdb_replaywin=%d)\n",
			seq, tdbp->tdb_replaywin_lastseq, diff, tdbp->tdb_replaywin);
		goto unlock_out_check;
	}
	retval = 1;			/* out of order but good */

unlock_out_check:
	spin_unlock(&tdbp->lock);

	return retval;
}

static int
ipsec_updatereplaywindow(struct tdb*tdbp, __u32 seq)
{
	int retval;
	__u32 diff;

	spin_lock(&tdbp->lock);
	
	if (tdbp->tdb_replaywin == 0) {	/* replay shut off */
		retval = 1;
		goto unlock_out_update;
	}
	if (seq == 0) {
		retval = 0;		/* first == 0 or wrapped */
		KLIPS_PRINT(debug_rcv & DB_RX_REPLAY,
			"ipsec_rcv: updatereplaywindow: seq == 0 (seq=%d)\n",
			seq);
		goto unlock_out_update;
	}

	/* new larger sequence number */
	if (seq > tdbp->tdb_replaywin_lastseq) {
		diff = seq - tdbp->tdb_replaywin_lastseq;

		/* In win, set bit for this pkt */
		if (diff < tdbp->tdb_replaywin)
			tdbp->tdb_replaywin_bitmap =
				(tdbp->tdb_replaywin_bitmap << diff) | 1;
		else
			/* This packet has way larger seq num */
			tdbp->tdb_replaywin_bitmap = 1;

		if(seq - tdbp->tdb_replaywin_lastseq - 1 > tdbp->tdb_replaywin_maxdiff) {
			tdbp->tdb_replaywin_maxdiff = seq - tdbp->tdb_replaywin_lastseq - 1;
		}
		tdbp->tdb_replaywin_lastseq = seq;
		retval = 1;		/* larger is good */
		goto unlock_out_update;
	}
	diff = tdbp->tdb_replaywin_lastseq - seq;

	/* too old or wrapped */ /* if wrapped, kill off SA? */
	if (diff >= tdbp->tdb_replaywin) {
/*
		if(seq < 0.25*max && tdbp->tdb_replaywin_lastseq > 0.75*max) {
			markdel_tdbchain(tdbp);
		}
*/	
		retval = 0;
		KLIPS_PRINT(debug_rcv & DB_RX_REPLAY,
			"ipsec_rcv: updatereplaywindow: diff >= tdbp->tdb_replaywin\n",
			"(seq=%d, tdb_replaywin_lastseq=%d, diff=%d, tdb_replaywin=%d)\n",
			seq, tdbp->tdb_replaywin_lastseq, diff, tdbp->tdb_replaywin);
		goto unlock_out_update;
	}
	/* this packet already seen */
	if (tdbp->tdb_replaywin_bitmap & (1LL << diff)) {
		retval = 0;
		KLIPS_PRINT(debug_rcv & DB_RX_REPLAY,
			"ipsec_rcv: updatereplaywindow: tdbp->tdb_replaywin_bitmap & (1LL << diff)\n",
			"(seq=%d, tdb_replaywin_lastseq=%d, diff=%d, tdb_replaywin=%d)\n",
			seq, tdbp->tdb_replaywin_lastseq, diff, tdbp->tdb_replaywin);
		goto unlock_out_update;
	}

	tdbp->tdb_replaywin_bitmap |= (1LL << diff);	/* mark as seen */
	retval = 1;			/* out of order but good */

unlock_out_update:
	spin_unlock(&tdbp->lock);

	return retval;
}

int
ipsec_rcv(struct sk_buff *skb)
{
#ifdef CONFIG_IPSEC_DEBUG
	struct net_device *dev = skb->dev;
#endif /* CONFIG_IPSEC_DEBUG */
	unsigned char protoc;
	struct iphdr *ipp;
	struct esp *espp = NULL;
	int esphlen = 0, authlen = 0;
	__u32 iv[4];
	unsigned char *ivp;
	CRYPTO_STATUS cs = CRYPTO_OK;
	unsigned char *authenticator = NULL;

	int hard_header_len;
	int iphlen;
	unsigned char *dat;
	struct tdb *tdbp, *tdbs;
	struct sa_id said;
	struct net_device_stats *stats = NULL;		/* This device's statistics */
	struct device *ipsecdev = NULL, *prvdev;
	struct ipsecpriv *prv;
	char name[9];
	char sa[SATOA_BUF];
	size_t sa_len;
	char ipaddr_txt[ADDRTOA_BUF];
	int i;
	int ike_packet = 0;
	struct in_addr ipaddr;
	__u8 next_header = 0;
	__u8 proto;
	
	int pad = 0, ilen, padlen = 0, len, replay = 0;
	__u8 *idat;
	struct tdb* tdbprev = NULL;
	struct tdb* tdbnext = NULL;

	unsigned int hid;

	tdbs = NULL;
	hid = 0;

	/* Don't unlink in the middle of a turnaround */
	MOD_INC_USE_COUNT;
	
	KLIPS_PRINT(debug_netlink, "ipsec_rcv: before read_lock\n");
	read_lock(&ipsec_sync_lock);
	KLIPS_PRINT(debug_netlink, "ipsec_rcv: after read_lock\n");

	ipsec_sync_lock_hd = 3;

	if (skb == NULL) {
		KLIPS_PRINT(debug_rcv, 
			    "klips_debug:ipsec_rcv: "
			    "NULL skb passed in.\n");
		goto rcvleave;
	}
		
#ifdef CONFIG_SMP
	hid = smp_processor_id();
#endif
#ifdef CONFIG_IPSECTHREAD
	hid = IPSEC_SKB_CB(skb)->hsips_hid;
#endif

	if (skb->data == NULL) {
		KLIPS_PRINT(debug_rcv,
			    "klips_debug:ipsec_rcv: "
			    "NULL skb->data passed in, packet is bogus, dropping.\n");
		goto rcvleave;
	}
	
	if (skb->nh.iph) { 
	        int ihl = skb->nh.iph->ihl * 4;

		skb->len += ihl;
		skb->data -= ihl;

		skb->nh.raw = skb->data;
	} else {
		KLIPS_PRINT(debug_rcv, "klips_debug:ipsec_rcv: "
		                       "can't see the ip-header.\n");
		goto rcvleave;
	}

	/* dev->hard_header_len is unreliable and should not be used */
	hard_header_len = skb->mac.raw ? (skb->data - skb->mac.raw) : 0;
	if((hard_header_len < 0) || (hard_header_len > skb_headroom(skb)))
		hard_header_len = 0;

	/* if skb was cloned (most likely due to a packet sniffer such as
	   tcpdump being momentarily attached to the interface), make
	   a copy of our own to modify */
	if(skb_cloned(skb)) {
		/* include any mac header while copying.. */
		if(skb_headroom(skb) < hard_header_len) {
			printk(KERN_WARNING "klips_error:ipsec_rcv: "
			       "tried to skb_push hhlen=%d, %d available.  "
			       "This should never happen, please report.\n",
			       hard_header_len, skb_headroom(skb));
			goto rcvleave;
		}
		skb_push(skb, hard_header_len);

		if ( skb_cow(skb, skb_headroom(skb)) ) {
			goto rcvleave;
		}
		if(skb->len < hard_header_len) {
			printk(KERN_WARNING "klips_error:ipsec_rcv: "
			       "tried to skb_pull hhlen=%d, %d available.  "
			       "This should never happen, please report.\n",
			       hard_header_len, skb->len);
			goto rcvleave;
		}
		skb_pull(skb, hard_header_len);
	}
	
	KLIPS_PRINT(debug_rcv, 
		    "klips_debug:ipsec_rcv: Info -- ");
	KLIPS_PRINTMORE(debug_rcv && skb->dev, "sdb->dev=%s ",
		    skb->dev->name ? skb->dev->name : "NULL");
	KLIPS_PRINTMORE(debug_rcv && dev, "dev=%s ",
		    dev->name ? dev->name : "NULL");
	KLIPS_PRINTMORE(debug_rcv, "\n");

	KLIPS_PRINT(debug_rcv && !(skb->dev && dev && (skb->dev == dev)),
		    "klips_debug:ipsec_rcv: Informational -- "
		    "**if this happens, find out why** "
		    "skb->dev:%s is not equal to dev:%s\n",
		    skb->dev ? (skb->dev->name ? skb->dev->name : "NULL") : "NULL",
		    dev ? (dev->name ? dev->name : "NULL") : "NULL");

	protoc = ((struct iphdr *)skb->data)->protocol;

	if( (protoc != IPPROTO_AH) &&
	    (protoc != IPPROTO_ESP) ) {
		KLIPS_PRINT(debug_rcv & DB_RX_TDB,
			    "klips_debug:ipsec_rcv: Why the hell is someone "
			    "passing me a non-ipsec packet? -- dropped.\n");
		goto rcvleave;
	}

	if(skb->dev) {
		for(i = 0; i < IPSEC_NUM_IF; i++) {
			sprintf(name, "ipsec%d", i);
			if(!strcmp(name, skb->dev->name)) {
				prv = (struct ipsecpriv *)(skb->dev->priv);
				if(prv) {
					stats = (struct net_device_stats *) &(prv->mystats);
				}
				ipsecdev = skb->dev;
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: Info -- pkt "
					    "already proc'ed a group of ipsec headers, "
					    "processing next group of ipsec headers.\n");
				break;
			}
			if((ipsecdev = ipsec_dev_get(name)) == NULL) {
				KLIPS_PRINT(debug_rcv,
					    "klips_error:ipsec_rcv: device %s does "
					    "not exist\n", name);
			}
			prv = ipsecdev ? (struct ipsecpriv *)(ipsecdev->priv) : NULL;
			prvdev = prv ? (struct device *)(prv->dev) : NULL;
			
			KLIPS_PRINT(debug_rcv && prvdev, 
				    "klips_debug:ipsec_rcv: physical device"
				    " for device %s is %s\n",
				    name, prvdev->name);
			if(prvdev && skb->dev &&
			   !strcmp(prvdev->name, skb->dev->name)) {
				stats = prv ? ((struct net_device_stats *) &(prv->mystats)) : NULL;
				skb->dev = ipsecdev;
				if(stats) {
					stats->rx_packets++;
				}
				break;
			}
		}
	} else {
		KLIPS_PRINT(debug_rcv, 
			    "klips_debug:ipsec_rcv: device supplied"
			    " with skb is NULL\n");
	}
			
	if(!stats) {
		ipsecdev = NULL;
	}
	KLIPS_PRINT((debug_rcv && !stats),
		    "klips_error:ipsec_rcv: packet received from physical "
		    "I/F (%s) not connected to ipsec I/F.  Cannot record "
		    "stats.\n"
		    "klips_error          May not have SA for decoding.  "
		    "Is IPSEC traffic expected on this I/F?  "
		    "Check routing.\n",
		    skb->dev ? (skb->dev->name ? skb->dev->name : "NULL") : "NULL");

#ifdef CONFIG_IPSEC_DEBUG
	if (debug_rcv) {
		struct timeval tv;
		do_gettimeofday(&tv);
		printk("klips_debug:ipsec_rcv: ts=%02ld.%04ld %s packet received,"
		       " dev = %s\n", 
		       (unsigned long int)tv.tv_sec % 60,
		       (unsigned long int)tv.tv_usec / 100,
		       protoc == IPPROTO_AH ? "AH" : "ESP",
		       ipsecdev ? ipsecdev->name : "NULL");
	}
#endif /* CONFIG_IPSEC_DEBUG */

	/* begin decapsulating loop here */
	do {
		espp = NULL;
		esphlen = 0;
		authlen = 0;

		len = skb->len;
		dat = skb->data;
		ipp = (struct iphdr *)skb->data;
		proto = ipp->protocol;
		ipaddr.s_addr = ipp->saddr;
		addrtoa(ipaddr, 0, ipaddr_txt, sizeof(ipaddr_txt));
		
		KLIPS_PRINT(debug_rcv, "klips_debug:ipsec_rcv: old\n");
		KLIPS_IP_PRINT(debug_rcv, ipp);
		
		iphlen = ipp->ihl << 2;
		ipp->check = 0;			/* we know the sum is good */
		
		if ((proto == IPPROTO_ESP) && ((len - iphlen) % 4)) {
			printk("klips_error:ipsec_rcv: got packet with content length = %d from "
			       "%s -- should be on 4 octet boundary, packet dropped\n",
			       len - iphlen, ipaddr_txt);	/* XXX this will need to be 8 for IPv6 */
			if(stats) {
				stats->rx_errors++;
			}
			goto rcvleave;
		}
		
		/*
		 * Find tunnel control block and (indirectly) call the appropriate
		 * tranform routine. The resulting sk_buf is a valid
		 * IP packet ready to go through input processing.
		 */
		
		said.dst.s_addr = ipp->daddr;
		switch(proto) {

		case IPPROTO_ESP:
			espp = (struct esp *)(skb->data + iphlen);
			said.spi = espp->esp_spi;
			break;

		default:
			if(stats) {
				stats->rx_errors++;
			}
			goto rcvleave;
		}
		said.proto = proto;
		
		if (atomic_read(&cug)) {
			if (ntohl(said.spi) == 1) {
			  said.dst.s_addr = 0;
			  ike_packet = 1;
			}
		}

		sa_len = satoa(said, 0, sa, SATOA_BUF);

		/*
		 * The spinlock is to prevent any other process from accessing or deleting
		 * the structure while we are using and updating it.
		 */
		tdbs = tdbp = __find_tdb(&said);

		sa_len = satoa(said, 0, sa, SATOA_BUF);
		if (tdbp == NULL || atomic_read(&tdbp->del_flag)) {		     
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: no Tunnel Descriptor "
				    "Block for SA:%s: incoming packet with no SA "
				    "dropped\n", sa);
			if(stats) {
				stats->rx_dropped++;
			}
			goto rcvleave;
		}
	
		if (sysctl_ipsec_inbound_policy_check) {
			if(ipp->saddr != ((struct sockaddr_in*)(tdbp->tdb_addr_s))->sin_addr.s_addr) {
				ipaddr.s_addr = ((struct sockaddr_in*)(tdbp->tdb_addr_s))->sin_addr.s_addr;
				addrtoa(ipaddr, 0, ipaddr_txt, sizeof(ipaddr_txt));
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "tdb src addr=%s", ipaddr_txt);
				ipaddr.s_addr = ipp->saddr;
				addrtoa(ipaddr, 0, ipaddr_txt, sizeof(ipaddr_txt));
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "SA:%s, src=%s of pkt does not agree with expected SA source address policy.\n",
					    sa, ipaddr_txt);
					     
				if(stats) {
					stats->rx_dropped++;
				}
				goto rcvleave;
			}
			{
				ipaddr.s_addr = ipp->saddr;
				addrtoa(ipaddr, 0, ipaddr_txt, sizeof(ipaddr_txt));
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "SA:%s, src=%s of pkt agrees with expected SA source address policy.\n",
					    sa, ipaddr_txt);
			}
			if(tdbnext) {
				if(tdbnext != tdbp) {
					KLIPS_PRINT(debug_rcv,
						    "klips_debug:ipsec_rcv: "
						    "unexpected SA:%s: does not agree with tdb->inext policy, dropped\n",
						    sa);
					if(stats) {
						stats->rx_dropped++;
					}
					goto rcvleave;
				}
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "SA:%s grouping from previous SA is OK.\n",
					    sa);
			} else {
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "SA:%s First SA in group.\n",
					    sa);
			}
			
			if(tdbp->tdb_onext) {
				if(tdbprev != tdbp->tdb_onext) {
					KLIPS_PRINT(debug_rcv,
						    "klips_debug:ipsec_rcv: "
						    "unexpected SA:%s: does not agree with tdb->onext policy, dropped.\n",
						    sa);
					if(stats) {
						stats->rx_dropped++;
					}
					goto rcvleave;
				} else {
					KLIPS_PRINT(debug_rcv,
						    "klips_debug:ipsec_rcv: "
						    "SA:%s grouping to previous SA is OK.\n",
						    sa);
				}
			} else {
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "SA:%s No previous backlink in group.\n",
					    sa);
			}
		}
		
		/* If it is in larval state, drop the packet, we cannot process yet. */
		if(tdbp->tdb_state == SADB_SASTATE_LARVAL) {
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: "
				    "TDB in larval state, cannot be used yet, "
				    "dropping packet.\n");
			if(stats) {
				stats->rx_dropped++;
			}
			goto rcvleave;
		}
		
		if(tdbp->tdb_state == SADB_SASTATE_DEAD) {
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: "
				    "TDB in dead state, cannot be used any more, "
				    "dropping packet.\n");
			if(stats) {
				stats->rx_dropped++;
			}
			goto rcvleave;
		}
		
		if(tdbp->tdb_lifetime_bytes_h &&
		   (tdbp->tdb_lifetime_bytes_c > tdbp->tdb_lifetime_bytes_h)) {
			pfkey_expire(tdbp, 1);
			markdel_tdbchain(tdbp);
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: "
				    "hard bytes lifetime of SA:%s has been reached, "
				    "SA expired, incoming packet dropped.\n", sa);
			if(stats) {
				stats->rx_dropped++;
			}
			goto rcvleave;
		}
		if(tdbp->tdb_lifetime_bytes_s &&
		   (tdbp->tdb_lifetime_bytes_c > tdbp->tdb_lifetime_bytes_s)) {
		   	if (tdbp->tdb_state != SADB_SASTATE_DYING)
		   	{
			tdbp->tdb_state = SADB_SASTATE_DYING;
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: "
				    "soft bytes lifetime of SA:%s has been reached, "
				    "SA expiring, soft expire message sent up, "
				    "incoming packet still processed.\n", sa);
			pfkey_expire(tdbp, 0);
			}
		}
		
		if(tdbp->tdb_lifetime_addtime_h &&
		   ((jiffies / HZ) - tdbp->tdb_lifetime_addtime_c >
		    tdbp->tdb_lifetime_addtime_h)) {
			pfkey_expire(tdbp, 1);
			markdel_tdbchain(tdbp);
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: "
				    "hard addtime lifetime of SA:%s has been reached, "
				    "SA expired, incoming packet dropped.\n", sa);
			if(stats) {
				stats->rx_dropped++;
			}
			goto rcvleave;
		}
		if(tdbp->tdb_lifetime_addtime_s &&
		   ((jiffies / HZ) - tdbp->tdb_lifetime_addtime_c >
		    tdbp->tdb_lifetime_addtime_s)) {
		       	if (tdbp->tdb_state != SADB_SASTATE_DYING)
		   	{
			tdbp->tdb_state = SADB_SASTATE_DYING;
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: "
				    "soft addtime lifetime of SA:%s has been reached, "
				    "SA expiring, soft expire message sent up, "
				    "incoming packet still processed.\n", sa);
			pfkey_expire(tdbp, 0);
			}
		}
		
		if(tdbp->tdb_lifetime_usetime_c) {
			if(tdbp->tdb_lifetime_usetime_h &&
			   ((jiffies / HZ) - tdbp->tdb_lifetime_usetime_c >
			    tdbp->tdb_lifetime_usetime_h)) {
				pfkey_expire(tdbp, 1);
				markdel_tdbchain(tdbp);
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "hard usetime lifetime of SA:%s has been reached, "
					    "SA expired, incoming packet dropped.\n", sa);
				if(stats) {
					stats->rx_dropped++;
				}
				goto rcvleave;
			}
			if(tdbp->tdb_lifetime_usetime_s &&
			   ((jiffies / HZ) - tdbp->tdb_lifetime_usetime_c >
			    tdbp->tdb_lifetime_usetime_s)) {
			       	if (tdbp->tdb_state != SADB_SASTATE_DYING)
		   		{
				tdbp->tdb_state = SADB_SASTATE_DYING;
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "soft usetime lifetime of SA:%s has been reached, "
					    "SA expiring, soft expire message sent up, "
					    "incoming packet still processed.\n", sa);
				pfkey_expire(tdbp, 0);
				}
			}
		}
		
		if(tdbp->tdb_lifetime_packets_h &&
		   (tdbp->tdb_lifetime_packets_c > tdbp->tdb_lifetime_packets_h)) {
			pfkey_expire(tdbp, 1);
			markdel_tdbchain(tdbp);
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: "
				    "hard packets lifetime of SA:%s has been reached, "
				    "SA expired, incoming packet dropped.\n", sa);
			if(stats) {
				stats->rx_dropped++;
			}
			goto rcvleave;
		}
		if(tdbp->tdb_lifetime_packets_s &&
		   (tdbp->tdb_lifetime_packets_c > tdbp->tdb_lifetime_packets_s)) {
		      	if (tdbp->tdb_state != SADB_SASTATE_DYING)
	   		{
			tdbp->tdb_state = SADB_SASTATE_DYING;
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: "
				    "soft packets lifetime of SA:%s has been reached, "
				    "SA expiring, soft expire message sent up, "
				    "incoming packet still processed.\n", sa);
			pfkey_expire(tdbp, 0);
			}
		}
		
		/* authenticate, if required */
		idat = dat + iphlen;

		switch(tdbp->tdb_authalg) {
		
		case AH_NONE:
			authlen = 0;
			break;
		
		default:
			authlen = AHHMAC_HASHLEN;
			break;
		}
		
		ilen = len - iphlen - authlen;
		
		if (ilen <= 0) {
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: short packet - length zero or negative %s,"
				    " packet dropped\n",
				    ipaddr_txt);
			if(stats) {
				stats->rx_dropped++;
			}
			goto rcvleave;
		}
		
		ivp = (__u8 *)espp+sizeof(struct esp);
		
		KLIPS_PRINT(proto == IPPROTO_ESP && debug_rcv, 
			    "klips_debug:ipsec_rcv: packet from %s received with"
			    " seq=%d (iv)=0x%08x%08x... iplen=%d esplen=%d sa=%s\n",
			    ipaddr_txt,
			    (__u32)ntohl(espp->esp_rpl),
			    (__u32)ntohl(*((__u32 *)ivp    )),
			    (__u32)ntohl(*((__u32 *)ivp + 1)),
			    len, ilen, sa);
		
		switch(proto) {

		case IPPROTO_ESP:
			replay = ntohl(espp->esp_rpl);
			authenticator = &(dat[len - authlen]);
			break;
		}

		if (!ipsec_checkreplaywindow(tdbp, replay)) {
			KLIPS_PRINT(debug_rcv & DB_RX_REPLAY,
				    "klips_debug:ipsec_rcv: duplicate frame from %s,"
				    " packet dropped\n",
				    ipaddr_txt);
			if(stats) {
				stats->rx_dropped++;
			}
			tdbp->tdb_replaywin_errs += 1;
			goto rcvleave;
		}
		
		/*
		 * verify authenticator in AH (ESP-Auth is done later)
		 */
		
		KLIPS_PRINT(debug_rcv,
			    "klips_debug:ipsec_rcv: encalg = %d, authalg = %d.\n",
			    tdbp->tdb_encalg, tdbp->tdb_authalg);

 	    	if(!authenticator) {
			if(stats) {
				stats->rx_dropped++;
			}
			tdbp->tdb_auth_errs += 1;
			goto rcvleave;
		}		

		if (proto == IPPROTO_ESP)
		{
		    	SData In, In_h, Out_e, Out_a, mi, **ppOut, *aOut[2];
			In.nKEK = In_h.nKEK = \
			Out_e.nKEK = Out_a.nKEK = mi.nKEK = 0;
		    	
			memcpy(iv, ivp, tdbp->tdb_iv_size);

#ifdef CONFIG_DEBUG_CRYPT	
			if (!ike_packet)
				dmp("IV used:", (caddr_t)iv, tdbp->tdb_iv_size);
#endif

			esphlen = sizeof(struct esp) + tdbp->tdb_iv_size;
			idat += esphlen;
			ilen -= esphlen;
			
			if ((ilen) % 8) {
				printk("klips_error:ipsec_rcv: got packet with esplen = %d from "
				       "%s -- should be on 8 octet boundary, packet dropped\n",
				       ilen, ipaddr_txt);
				if(stats) {
					stats->rx_errors++;
				}
				tdbp->tdb_encsize_errs += 1;
				goto rcvleave;
			}

		    	In.nDataLength = ilen;
			In.pData = idat;

#ifdef CONFIG_DEBUG_CRYPT
			if (!ike_packet)
				dmp("Packet to decrypt:", In.pData, In.nDataLength);
#endif
			    
			Out_e.nDataLength = ilen;
			Out_e.pData = kmalloc(ilen, GFP_ATOMIC);
			if (Out_e.pData == NULL)  {
			      printk(KERN_WARNING "klips_error:ipsec_tunnel_start_xmit: no MEM");
			      goto rcvleave;
			}
				    				
			mi.pData = iv;
			mi.nDataLength = tdbp->tdb_iv_size;

			if (tdbp->tdb_authalg)
			{
				In_h.nDataLength = sizeof(struct esp) + tdbp->tdb_iv_size;
				In_h.pData = (caddr_t)espp;				  
				    
				Out_a.nDataLength = authlen;
				Out_a.pData = authenticator;
				
				aOut[0] = &Out_a;
				aOut[1] = &Out_e;
				ppOut = (SData **) &aOut;
				
				if (!tdbp->tdb_i_combined_handle[hid]) {
				      KLIPS_PRINT(debug_tunnel, "need a combined handle here\n");
		              	      kfree(Out_e.pData);
		              	      goto rcvleave;
				}

#ifdef CONFIG_DEBUG_CRYPT
				if (!ike_packet)
					dmp("Additional Header to MAC:", In_h.pData, In_h.nDataLength);

				if (!ike_packet)
					dmp("Received MAC:", authenticator, authlen);
#endif

				if (!tdbp->tdb_is_pepp) {
				  cs = crypto_transform(tdbp->tdb_authalg_v_handle[hid], &In_h, NULL, NULL);
				  if (cs != CRYPTO_OK) {
				        KLIPS_PRINT(debug_tunnel, "error crypto transform #%d\n",  cs);
				        cs = crypto_reopen(tdbp->tdb_authalg_v_handle[hid], NULL);
		              	        kfree(Out_e.pData);
		              	        goto rcvleave;
				  }
				}	

				cs = crypto_c_transform(tdbp->tdb_i_combined_handle[hid], &In, (SData **) ppOut, 2, &mi);
				
				crypto_reopen(tdbp->tdb_authalg_v_handle[hid], NULL);
				
				if (cs == CRYPTO_ERR_VERIFICATION_FAILED)
	             		{
	 				KLIPS_PRINT(debug_rcv & DB_RX_INAU,
					    "klips_debug:ipsec_rcv: auth failed on incoming packet from %s, dropped\n", ipaddr_txt);
					
					printk("klips_debug:ipsec_rcv: auth failed on incoming packet dropped\n");

					if(stats) {
						stats->rx_dropped++;
					}
					tdbp->tdb_auth_errs += 1;
					kfree(Out_e.pData);
					goto rcvleave;
				} 
				else if (cs != CRYPTO_OK) 
				{
				      KLIPS_PRINT(debug_tunnel, "error crypto transform #%d\n",  cs);
		              	      kfree(Out_e.pData);
		              	      goto rcvleave;
				}
				else {
					KLIPS_PRINT(debug_rcv,
				    	"klips_debug:ipsec_rcv: authentication successful.\n");
				}

			}
			else
			{	
			    cs = crypto_transform(tdbp->tdb_encalg_d_handle[hid], &In, &Out_e, &mi);
			    if (cs != CRYPTO_OK) {
			      KLIPS_PRINT(debug_tunnel, "error crypto transform #%d\n",  cs);
			      kfree(Out_e.pData);
			      goto rcvleave;
			    }
			}

#ifdef CONFIG_DEBUG_CRYPT
			if (!ike_packet)
				dmp("Decrypted Packet:", Out_e.pData, Out_e.nDataLength);
#endif
			if (tdbp->tdb_is_pepp) {
			  int failed=0;
			  if (Out_e.nDataLength<=PEPP_HEADER_FULL_SIZE)
			    failed=1;
			  else 
 			    if (memcmp(espp,Out_e.pData,PEPP_HEADER_SHORT_SIZE)!=0) {
			 	failed=1;
			    } else {
			      int len=s_to_i((caddr_t)Out_e.pData+PEPP_HEADER_SHORT_SIZE);
			      if (len!=Out_e.nDataLength) 
			  	failed=1;
			    } 

			  if (failed) {
				KLIPS_PRINT(debug_rcv & DB_RX_INAU,
				    "klips_debug:ipsec_rcv: auth failed on incoming packet from %s, dropped\n", ipaddr_txt);
				
				printk("klips_debug:ipsec_rcv: auth failed on incoming packet dropped\n");

				if(stats) {
					stats->rx_dropped++;
				}
				tdbp->tdb_auth_errs += 1;
				kfree(Out_e.pData);
				goto rcvleave;
			  }
			  ilen=Out_e.nDataLength-PEPP_HEADER_FULL_SIZE;
			  memcpy(idat,(caddr_t)Out_e.pData+PEPP_HEADER_FULL_SIZE,
				ilen);
			  skb_trim(skb,skb->len-PEPP_HEADER_FULL_SIZE);
			} else {
			  memcpy(idat, Out_e.pData, Out_e.nDataLength);
			}
			kfree(Out_e.pData);
			    
			next_header = idat[ilen - 1];
			padlen = idat[ilen - 2];
			pad = padlen + 2 + authlen;
			{
			        int badpad = 0;
				
				KLIPS_PRINT(debug_rcv & DB_RX_IPAD,
					    "klips_debug:ipsec_rcv: "
					    "padlen=%d, contents: 0x<offset>: 0x<value> 0x<value> ...\n",
					    padlen);
				
				for (i = 1; i <= padlen; i++) {
					if((i % 16) == 1) {
						KLIPS_PRINT(debug_rcv & DB_RX_IPAD,
							    "klips_debug:           %02x:",
							    i - 1);
					}
					KLIPS_PRINTMORE(debug_rcv & DB_RX_IPAD,
						    " %02x",
						    idat[ilen - 2 - padlen + i - 1]);
					if(i != idat[ilen - 2 - padlen + i - 1]) {
					        badpad = 1;
					} 
					if((i % 16) == 0) {
						KLIPS_PRINTMORE(debug_rcv & DB_RX_IPAD,
							    "\n");
					}
				}
				if((i % 16) != 1) {
					KLIPS_PRINTMORE(debug_rcv & DB_RX_IPAD,
							"\n");
				}
				if(badpad) {
					KLIPS_PRINT(debug_rcv & DB_RX_IPAD,
						    "klips_debug:ipsec_rcv: "
						    "warning, decrypted packet from %s has bad padding\n",
						    ipaddr_txt);
					KLIPS_PRINT(debug_rcv & DB_RX_IPAD,
						    "klips_debug:ipsec_rcv: "
						    "...may be bad decryption -- not dropped\n");
					tdbp->tdb_encpad_errs += 1;
				}
				
				KLIPS_PRINT(debug_rcv & DB_RX_IPAD,
					    "klips_debug:ipsec_rcv: "
					    "packet decrypted from %s: next_header = %d, padding = %d\n",
					    ipaddr_txt,
					    next_header,
					    pad - 2 - authlen);
			}
		}

		/* If the sequence number == 0, expire SA, it had rolled */
		if(tdbp->tdb_replaywin && !replay /* !tdbp->tdb_replaywin_lastseq */) {
			markdel_tdbchain(tdbp);
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_tunnel_start_xmit: "
				    "replay window counter rolled, expiring SA.\n");
			if(stats) {
				stats->rx_dropped++;
			}
			goto rcvleave;
		}

		if (!ipsec_updatereplaywindow(tdbp, replay)) {
			KLIPS_PRINT(debug_rcv & DB_RX_REPLAY,
				    "klips_debug:ipsec_rcv: duplicate frame from %s,"
				    " packet dropped\n",
				    ipaddr_txt);
			if(stats) {
				stats->rx_dropped++;
			}
			tdbp->tdb_replaywin_errs += 1;
			goto rcvleave;
		}
		
	       
		tdbp->tdb_lifetime_bytes_c += ilen;
		if(!tdbp->tdb_lifetime_usetime_c) {
			tdbp->tdb_lifetime_usetime_c = jiffies / HZ;
		}
		tdbp->tdb_lifetime_usetime_l = jiffies / HZ;
		tdbp->tdb_lifetime_packets_c += 1;
		

		/*
		 *	Discard the original IP header
		 */
		
		KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
			    "klips_debug:ipsec_rcv: proto = %d, protoc = %d, "
			    "len = %d.\n",
			    proto, protoc, len);

		ipp->protocol = next_header;
		
		switch(proto) {

		case IPPROTO_ESP:
			ipp->tot_len = htons(ntohs(ipp->tot_len) - (esphlen + pad));
			memmove((void *)(skb->data + esphlen),
				(void *)(skb->data), iphlen);
			if(skb->len < esphlen) {
				printk(KERN_WARNING "klips_error:ipsec_rcv: "
				       "tried to skb_pull esphlen=%d, %d available.  "
				       "This should never happen, please report.\n",
				       esphlen, (int)(skb->len));
				goto rcvleave;
			}
			skb_pull(skb, esphlen);
		break;
		}

		if(proto == IPPROTO_ESP) {
			KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
				    "klips_debug:ipsec_rcv: trimming to %d.\n",
				    len - esphlen - pad);
			if(pad + esphlen <= len) {
			        skb_trim(skb, len - esphlen - pad);
			} else {
			        KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
				            "klips_debug:ipsec_rcv: bogus packet, size is zero or negative, dropping.\n");
				goto rcvleave;
			}
		}
		
		/*
		 *	Adjust pointers
		 */
		
		len = skb->len;
		dat = skb->data;
		
		skb->nh.raw = skb->data;
		skb->h.raw = skb->nh.raw + (skb->nh.iph->ihl << 2);
		
		ipp = (struct iphdr *)dat;
		ipp->check = 0;
		ipp->check = ip_fast_csum((unsigned char *)dat, iphlen >> 2);

		KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
			    "klips_debug:ipsec_rcv: after <%s%s%s>, SA:%s:\n",
					    TDB_XFORM_NAME(tdbp), sa);
		KLIPS_IP_PRINT(debug_rcv & DB_RX_PKTRX, ipp);
		
		skb->protocol = htons(ETH_P_IP);
		skb->ip_summed = 0;

		tdbprev = tdbp;
		tdbnext = tdbp->tdb_inext;

		if(sysctl_ipsec_inbound_policy_check) {
			if(tdbnext) {
				if(tdbnext->tdb_onext != tdbp) {
					KLIPS_PRINT(debug_rcv,
						    "klips_debug:ipsec_rcv: "
						    "SA:%s, backpolicy does not agree with fwdpolicy.\n",
						    sa);
					if(stats) {
						stats->rx_dropped++;
					}
					goto rcvleave;
				}
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "SA:%s, backpolicy agrees with fwdpolicy.\n",
					    sa);
				if(!(   (ipp->protocol == IPPROTO_AH  )
					|| (ipp->protocol == IPPROTO_ESP )
					|| (ipp->protocol == IPPROTO_IPIP)
					)) {
					KLIPS_PRINT(debug_rcv,
						    "klips_debug:ipsec_rcv: "
						    "packet with incomplete policy dropped, last successful SA:%s.\n",
						    sa);
					if(stats) {
						stats->rx_dropped++;
					}
					goto rcvleave;
				}
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "SA:%s, Another IPSEC header to process.\n",
					    sa);
			} else {
				KLIPS_PRINT(debug_rcv,
					    "klips_debug:ipsec_rcv: "
					    "No tdb_inext from this SA:%s.\n",
					    sa);
			}
		}

	} while((ipp->protocol == IPPROTO_ESP) ||
		(ipp->protocol == IPPROTO_AH )
#ifdef CONFIG_IPSEC_IPCOMP
		|| (ipp->protocol == IPPROTO_COMP)
#endif /* CONFIG_IPSEC_IPCOMP */
		);
	/* end decapsulation loop here */
	

	/*
	 * XXX this needs to be locked from when it was first looked
	 * up in the decapsulation loop.  Perhaps it is better to put
	 * the IPIP decap inside the loop.
	 */
	if(tdbnext) {
		tdbp = tdbnext;
		sa_len = satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
		if(ipp->protocol != IPPROTO_IPIP) {
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: "
				    "SA:%s, Hey!  How did this get through?  Dropped.\n",
				    sa);
			if(stats) {
				stats->rx_dropped++;
			}
			goto rcvleave;
		}
		if(sysctl_ipsec_inbound_policy_check && (tdbnext = tdbp->tdb_inext)) {
			char sa2[SATOA_BUF];
			sa_len = satoa(tdbnext->tdb_said, 0, sa2, SATOA_BUF);
			KLIPS_PRINT(debug_rcv,
				    "klips_debug:ipsec_rcv: "
				    "unexpected SA:%s after IPIP SA:%s\n",
				    sa2, sa);
			if(stats) {
				stats->rx_dropped++;
			}
			goto rcvleave;
		}
	}

	if(ipp->protocol == IPPROTO_IPIP) {
		if(skb->len < iphlen) {
			printk(KERN_WARNING "klips_debug:ipsec_rcv: "
			       "tried to skb_pull iphlen=%d, %d available.  "
			       "This should never happen, please report.\n",
			       iphlen, (int)(skb->len));
			goto rcvleave;
		}
		skb_pull(skb, iphlen);

		ipp = (struct iphdr *)skb->nh.raw = skb->data;
		skb->h.raw = skb->nh.raw + (skb->nh.iph->ihl << 2);
		
		skb->protocol = htons(ETH_P_IP);
		skb->ip_summed = 0;
		KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
			    "klips_debug:ipsec_rcv: IPIP tunnel stripped.\n");
		KLIPS_IP_PRINT(debug_rcv & DB_RX_PKTRX, ipp);
	}

	if(stats) {
		stats->rx_bytes += skb->len;
	}
	if(skb->dst) {
		dst_release(skb->dst);
		skb->dst = NULL;
	}
	skb->pkt_type = PACKET_HOST;
	if(hard_header_len &&
	   (skb->mac.raw != (skb->data - hard_header_len)) &&
	   (hard_header_len <= skb_headroom(skb))) {
		/* copy back original MAC header */
		memmove(skb->data - hard_header_len, skb->mac.raw, hard_header_len);
		skb->mac.raw = skb->data - hard_header_len;
	}
#ifdef SKB_RESET_NFCT
        nf_conntrack_put(skb->nfct);
        skb->nfct = NULL;
#ifdef CONFIG_NETFILTER_DEBUG
	skb->nf_debug = 0;
#endif /* CONFIG_NETFILTER_DEBUG */
#endif /* SKB_RESET_NFCT */

#ifdef CONFIG_CLOSEIPSEC
	skb->ipsec_mark = 1;
#endif

	KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
		    "klips_debug:ipsec_rcv: netif_rx() called.\n");

rcv_unlock:
	MOD_DEC_USE_COUNT;

	ipsec_sync_lock_hd = 0;

	KLIPS_PRINT(debug_netlink, "ipsec_rcv: before read_unlock\n");
	read_unlock(&ipsec_sync_lock);
	wake_up(&ipsec_sync_lock_wq);
	KLIPS_PRINT(debug_netlink, "ipsec_rcv: after read_unlock\n");

 	if (skb) {
#ifdef CONFIG_IPSEC_PKT_SEQ
		hsips_netif_rx(skb);
#else
		KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
		            "klips_debug:ipsec_rcv: netif_rx() called.\n");

		memset(IPSEC_SKB_CB(skb), 0, sizeof(ipsec_cb_t));
		netif_rx(skb);
#endif
	}


	return 0;

rcvleave:
	if (skb) {
#ifdef CONFIG_IPSEC_PKT_SEQ
		IPSEC_SKB_CB(skb)->ignore = 1;
#else
		kfree_skb(skb);
		skb = NULL;
#endif
	}

	goto rcv_unlock;
}


static int __ipsec_rcv(struct sk_buff *skb)
{
	if (!skb)
		return 0;

	memset(skb->cb, 0, sizeof(ipsec_cb_t));

#ifdef CONFIG_IPSECTHREAD
	return hsips_rcv(skb);
#else
	return ipsec_rcv(skb);
#endif
}


struct inet_protocol ah_protocol =
{
	__ipsec_rcv,				/* AH handler */
	NULL,				/* TUNNEL error control */
	0,				/* next */
	IPPROTO_AH,			/* protocol ID */
	0,				/* copy */
	NULL,				/* data */
	"AH"				/* name */
};

struct inet_protocol esp_protocol = 
{
	__ipsec_rcv,			/* ESP handler          */
	NULL,				/* TUNNEL error control */
	0,				/* next */
	IPPROTO_ESP,			/* protocol ID */
	0,				/* copy */
	NULL,				/* data */
	"ESP"				/* name */
};

/*
 * Revision 1.56  2000/09/08 19:12:56  rgb
 * Change references from DEBUG_IPSEC to CONFIG_IPSEC_DEBUG.
 *
 * Revision 1.55  2000/08/28 18:15:46  rgb
 * Added MB's nf-debug reset patch.
 *
 * Revision 1.54  2000/08/27 01:41:26  rgb
 * More minor tweaks to the bad padding debug code.
 *
 * Revision 1.53  2000/08/24 16:54:16  rgb
 * Added KLIPS_PRINTMORE macro to continue lines without KERN_INFO level
 * info.
 * Tidied up device reporting at the start of ipsec_rcv.
 * Tidied up bad padding debugging and processing.
 *
 * Revision 1.52  2000/08/20 21:36:03  rgb
 * Activated pfkey_expire() calls.
 * Added a hard/soft expiry parameter to pfkey_expire().
 * Added sanity checking to avoid propagating zero or smaller-length skbs
 * from a bogus decryption.
 * Re-arranged the order of soft and hard expiry to conform to RFC2367.
 * Clean up references to CONFIG_IPSEC_PFKEYv2.
 *
 * Revision 1.51  2000/08/18 21:23:30  rgb
 * Improve bad padding warning so that the printk buffer doesn't get
 * trampled.
 *
 * Revision 1.50  2000/08/01 14:51:51  rgb
 * Removed _all_ remaining traces of DES.
 *
 * Revision 1.49  2000/07/28 13:50:53  rgb
 * Changed enet_statistics to net_device_stats and added back compatibility
 * for pre-2.1.19.
 *
 * Revision 1.48  2000/05/10 19:14:40  rgb
 * Only check usetime against soft and hard limits if the tdb has been
 * used.
 * Cast output of ntohl so that the broken prototype doesn't make our
 * compile noisy.
 *
 * Revision 1.47  2000/05/09 17:45:43  rgb
 * Fix replay bitmap corruption bug upon receipt of bogus packet
 * with correct SPI.  This was a DoS.
 *
 * Revision 1.46  2000/03/27 02:31:58  rgb
 * Fixed authentication failure printout bug.
 *
 * Revision 1.45  2000/03/22 16:15:37  rgb
 * Fixed renaming of dev_get (MB).
 *
 * Revision 1.44  2000/03/16 08:17:24  rgb
 * Hardcode PF_KEYv2 support.
 * Fixed minor bug checking AH header length.
 *
 * Revision 1.43  2000/03/14 12:26:59  rgb
 * Added skb->nfct support for clearing netfilter conntrack bits (MB).
 *
 * Revision 1.42  2000/01/26 10:04:04  rgb
 * Fixed inbound policy checking on transport mode bug.
 * Fixed noisy 2.0 printk arguments.
 *
 * Revision 1.41  2000/01/24 20:58:02  rgb
 * Improve debugging/reporting support for (disabled) inbound
 * policy checking.
 *
 * Revision 1.40  2000/01/22 23:20:10  rgb
 * Fixed up inboud policy checking code.
 * Cleaned out unused crud.
 *
 * Revision 1.39  2000/01/21 06:15:29  rgb
 * Added sanity checks on skb_push(), skb_pull() to prevent panics.
 * Fixed cut-and-paste debug_tunnel to debug_rcv.
 * Added inbound policy checking code, disabled.
 * Simplified output code by updating ipp to post-IPIP decapsulation.
 *
 * Revision 1.38  1999/12/22 05:08:36  rgb
 * Checked for null skb, skb->dev, skb->data, skb->dev->name, dev->name,
 * protocol and take appropriate action for sanity.
 * Set ipsecdev to NULL if device could not be determined.
 * Fixed NULL stats access bug if device could not be determined.
 *
 * Revision 1.37  1999/12/14 20:07:59  rgb
 * Added a default switch case to catch bogus encalg values.
 *
 * Revision 1.36  1999/12/07 18:57:57  rgb
 * Fix PFKEY symbol compile error (SADB_*) without pfkey enabled.
 *
 * Revision 1.35  1999/12/01 22:15:35  rgb
 * Add checks for LARVAL and DEAD SAs.
 * Change state of SA from MATURE to DYING when a soft lifetime is
 * reached and print debug warning.
 *
 * Revision 1.34  1999/11/23 23:04:03  rgb
 * Use provided macro ADDRTOA_BUF instead of hardcoded value.
 * Sort out pfkey and freeswan headers, putting them in a library path.
 *
 * Revision 1.33  1999/11/19 01:10:06  rgb
 * Enable protocol handler structures for static linking.
 *
 * Revision 1.32  1999/11/18 04:09:19  rgb
 * Replaced all kernel version macros to shorter, readable form.
 *
 * Revision 1.31  1999/11/17 15:53:39  rgb
 * Changed all occurrences of #include "../../../lib/freeswan.h"
 * to #include <freeswan.h> which works due to -Ilibfreeswan in the
 * klips/net/ipsec/Makefile.
 *
 * Revision 1.30  1999/10/26 15:09:07  rgb
 * Used debug compiler directives to shut up compiler for decl/assign
 * statement.
 *
 * Revision 1.29  1999/10/16 18:25:37  rgb
 * Moved SA lifetime expiry checks before packet processing.
 * Expire SA on replay counter rollover.
 *
 * Revision 1.28  1999/10/16 04:23:07  rgb
 * Add stats for replaywin_errs, replaywin_max_sequence_difference,
 * authentication errors, encryption size errors, encryption padding
 * errors, and time since last packet.
 *
 * Revision 1.27  1999/10/16 00:30:47  rgb
 * Added SA lifetime counting.
 *
 * Revision 1.26  1999/10/15 22:14:37  rgb
 * Add debugging.
 *
 * Revision 1.25  1999/10/08 18:37:34  rgb
 * Fix end-of-line spacing to sate whining PHMs.
 *
 * Revision 1.24  1999/10/03 18:54:51  rgb
 * Spinlock support for 2.3.xx.
 * Don't forget to undo spinlocks on error!
 *
 * Revision 1.23  1999/10/01 15:44:53  rgb
 * Move spinlock header include to 2.1> scope.
 *
 * Revision 1.22  1999/10/01 00:01:54  rgb
 * Added tdb structure locking.
 *
 * Revision 1.21  1999/09/18 11:42:12  rgb
 * Add Marc Boucher's tcpdump cloned packet fix.
 *
 * Revision 1.20  1999/09/17 23:50:25  rgb
 * Add Marc Boucher's hard_header_len patches.
 *
 * Revision 1.19  1999/09/10 05:31:36  henry
 * tentative fix for 2.0.38-crash bug (move chunk of new code into 2.2 #ifdef)
 *
 * Revision 1.18  1999/08/28 08:28:06  rgb
 * Delete redundant sanity check.
 *
 * Revision 1.17  1999/08/28 02:00:58  rgb
 * Add an extra sanity check for null skbs.
 *
 * Revision 1.16  1999/08/27 05:21:38  rgb
 * Clean up skb->data/raw/nh/h manipulation.
 * Add Marc Boucher's mods to aid tcpdump.
 *
 * Revision 1.15  1999/08/25 14:22:40  rgb
 * Require 4-octet boundary check only for ESP.
 *
 * Revision 1.14  1999/08/11 08:36:44  rgb
 * Add compiler directives to allow configuring out AH, ESP or transforms.
 *
 * Revision 1.13  1999/08/03 17:10:49  rgb
 * Cosmetic fixes and clarification to debug output.
 *
 * Revision 1.12  1999/05/09 03:25:36  rgb
 * Fix bug introduced by 2.2 quick-and-dirty patch.
 *
 * Revision 1.11  1999/05/08 21:23:57  rgb
 * Add casting to silence the 2.2.x compile.
 *
 * Revision 1.10  1999/05/05 22:02:31  rgb
 * Add a quick and dirty port to 2.2 kernels by Marc Boucher <marc@mbsi.ca>.
 *
 * Revision 1.9  1999/04/29 15:18:01  rgb
 * hange debugging to respond only to debug_rcv.
 * Change gettdb parameter to a pointer to reduce stack loading and
 * facilitate parameter sanity checking.
 *
 * Revision 1.8  1999/04/15 15:37:24  rgb
 * Forward check changes from POST1_00 branch.
 *
 * Revision 1.4.2.2  1999/04/13 20:32:45  rgb
 * Move null skb sanity check.
 * Silence debug a bit more when off.
 * Use stats more effectively.
 *
 * Revision 1.4.2.1  1999/03/30 17:10:32  rgb
 * Update AH+ESP bugfix.
 *
 * Revision 1.7  1999/04/11 00:28:59  henry
 * GPL boilerplate
 *
 * Revision 1.6  1999/04/06 04:54:27  rgb
 * Fix/Add RCSID Id: and Log: bits to make PHMDs happy.  This includes
 * patch shell fixes.
 *
 * Revision 1.5  1999/03/17 15:39:23  rgb
 * Code clean-up.
 * Bundling bug fix.
 * ESP_NULL esphlen and IV bug fix.
 *
 * Revision 1.4  1999/02/17 16:51:02  rgb
 * Ditch NET_IPIP dependancy.
 * Decapsulate recursively for an entire bundle.
 *
 * Revision 1.3  1999/02/12 21:22:47  rgb
 * Convert debugging printks to KLIPS_PRINT macro.
 * Clean-up cruft.
 * Process IPIP tunnels internally.
 *
 * Revision 1.2  1999/01/26 02:07:36  rgb
 * Clean up debug code when switched off.
 * Remove references to INET_GET_PROTOCOL.
 *
 * Revision 1.1  1999/01/21 20:29:11  rgb
 * Converted from transform switching to algorithm switching.
 *
 *
 * Id: ipsec_esp.c,v 1.16 1998/12/02 03:08:11 rgb Exp $
 *
 * Log: ipsec_esp.c,v $
 * Revision 1.16  1998/12/02 03:08:11  rgb
 * Fix incoming I/F bug in AH and clean up inconsistencies in the I/F
 * discovery routine in both AH and ESP.
 *
 * Revision 1.15  1998/11/30 13:22:51  rgb
 * Rationalised all the klips kernel file headers.  They are much shorter
 * now and won't conflict under RH5.2.
 *
 * Revision 1.14  1998/11/10 05:55:37  rgb
 * Add even more detail to 'wrong I/F' debug statement.
 *
 * Revision 1.13  1998/11/10 05:01:30  rgb
 * Clean up debug output to be quiet when disabled.
 * Add more detail to 'wrong I/F' debug statement.
 *
 * Revision 1.12  1998/10/31 06:39:32  rgb
 * Fixed up comments in #endif directives.
 * Tidied up debug printk output.
 * Convert to addrtoa and satoa where possible.
 *
 * Revision 1.11  1998/10/27 00:49:30  rgb
 * AH+ESP bundling bug has been squished.
 * Cosmetic brace fixing in code.
 * Newlines added before calls to ipsec_print_ip.
 * Fix debug output function ID's.
 *
 * Revision 1.10  1998/10/22 06:37:22  rgb
 * Fixed run-on error message to fit 80 columns.
 *
 * Revision 1.9  1998/10/20 02:41:04  rgb
 * Fixed a replay window size sanity test bug.
 *
 * Revision 1.8  1998/10/19 18:55:27  rgb
 * Added inclusion of freeswan.h.
 * sa_id structure implemented and used: now includes protocol.
 * \n bugfix to printk debug message.
 *
 * Revision 1.7  1998/10/09 04:23:03  rgb
 * Fixed possible DoS caused by invalid transform called from an ESP
 * packet.  This should not be a problem when protocol is added to the SA.
 * Sanity check added for null xf_input routine.  Sanity check added for null
 * socket buffer returned from xf_input routine.
 * Added 'klips_debug' prefix to all klips printk debug statements.
 *
 * Revision 1.6  1998/07/14 15:56:04  rgb
 * Set sdb->dev to virtual ipsec I/F.
 *
 * Revision 1.5  1998/06/30 18:07:46  rgb
 * Change for ah/esp_protocol stuct visible only if module.
 *
 * Revision 1.4  1998/06/30 00:12:46  rgb
 * Clean up a module compile error.
 *
 * Revision 1.3  1998/06/25 19:28:06  rgb
 * Readjust premature unloading of module on packet receipt.
 * Make protocol structure abailable to rest of kernel.
 * Use macro for protocol number.
 *
 * Revision 1.2  1998/06/23 02:49:34  rgb
 * Fix minor #include bug that prevented compiling without debugging.
 * Added code to check for presence of IPIP protocol if an incoming packet
 * is IPIP encapped.
 *
 * Revision 1.1  1998/06/18 21:27:44  henry
 * move sources from klips/src to klips/net/ipsec, to keep stupid
 * kernel-build scripts happier in the presence of symlinks
 *
 * Revision 1.9  1998/06/14 23:48:42  rgb
 * Fix I/F name comparison oops bug.
 *
 * Revision 1.8  1998/06/11 07:20:04  rgb
 * Stats fixed for rx_packets.
 *
 * Revision 1.7  1998/06/11 05:53:34  rgb
 * Added stats for rx error and good packet reporting.
 *
 * Revision 1.6  1998/06/05 02:27:28  rgb
 * Add rx_errors stats.
 * Fix DoS bug:  skb's not being freed on dropped packets.
 *
 * Revision 1.5  1998/05/27 21:21:29  rgb
 * Fix DoS potential bug.  skb was not being freed if the packet was bad.
 *
 * Revision 1.4  1998/05/18 22:31:37  rgb
 * Minor change in debug output and comments.
 *
 * Revision 1.3  1998/04/21 21:29:02  rgb
 * Rearrange debug switches to change on the fly debug output from user
 * space.  Only kernel changes checked in at this time.  radij.c was also
 * changed to temporarily remove buggy debugging code in rj_delete causing
 * an OOPS and hence, netlink device open errors.
 *
 * Revision 1.2  1998/04/12 22:03:19  rgb
 * Updated ESP-3DES-HMAC-MD5-96,
 * 	ESP-DES-HMAC-MD5-96,
 * 	AH-HMAC-MD5-96,
 * 	AH-HMAC-SHA1-96 since Henry started freeswan cvs repository
 * from old standards (RFC182[5-9] to new (as of March 1998) drafts.
 *
 * Fixed eroute references in /proc/net/ipsec*.
 *
 * Started to patch module unloading memory leaks in ipsec_netlink and
 * radij tree unloading.
 *
 * Revision 1.1  1998/04/09 03:05:59  henry
 * sources moved up from linux/net/ipsec
 *
 * Revision 1.1.1.1  1998/04/08 05:35:04  henry
 * RGB's ipsec-0.8pre2.tar.gz ipsec-0.8
 *
 * Revision 0.4  1997/01/15 01:28:15  ji
 * Minor cosmetic changes.
 *
 * Revision 0.3  1996/11/20 14:35:48  ji
 * Minor Cleanup.
 * Rationalized debugging code.
 *
 * Revision 0.2  1996/11/02 00:18:33  ji
 * First limited release.
 *
 *
 */
