/*
 * filename: hsips.h
 *
 * Developed by secunet AG
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */

#ifndef _SINA_HSIPS_H
#define _SINA_HSIPS_H

#include <linux/config.h>
#include <linux/netdevice.h>
#include <linux/skbuff.h>
#include <linux/list.h>
#include <asm/semaphore.h>

#define IPST_UNDEF	10
#define IPST_INIT	11
#define IPST_RUN	12

#define IPSEC_THREAD_QSIZE (netdev_max_backlog * smp_num_cpus * 10)

#define IPSEC_THREAD_RX_Q 1
#define IPSEC_THREAD_TX_Q 2

#define HSIPS_TX 0
#define HSIPS_RX 1

typedef struct hsips_queue_s {
	struct list_head queue;
	spinlock_t lock;
	atomic_t size;
} hsips_queue_t;

struct ipst_status {
	struct list_head list;
	struct list_head idle_list;

	struct task_struct *task;
	pid_t              pid;
	unsigned int       num;

	atomic_t           state;
	atomic_t           count;

	spinlock_t         lock;
	wait_queue_head_t  wq;
	struct semaphore   sem;
	atomic_t           up;
	atomic_t           dn;
	atomic_t           sig;

	void               *cu;

	int (*init) (void *);
	int (*ctl) (struct ipst_status *, void *);
	int (*destroy) (struct ipst_status *);
};

struct respawn_status {
	struct task_struct *task;
	pid_t              pid;

	unsigned int       num;

	atomic_t	   rx_drop;
	atomic_t	   tx_drop;
	atomic_t	   rx_err;
	atomic_t	   tx_err;
};

#define IPSEC_MAX_THREADS 32

extern struct respawn_status respawn_thread;

extern int hsips_init(void);
extern int hsips_cleanup(void);
extern int hsips_get_thread_cnt(void);

extern int hsips_get_qsize(void);
extern struct sk_buff *hsips_get_packet(void);

extern int hsips_start_xmit(struct sk_buff *, struct net_device *);
extern int hsips_rcv(struct sk_buff *);

extern void activate_thread(void);

extern int hsips_respawn(void *status_p);
extern int hsips_destroy(struct respawn_status *status);

extern struct list_head hsips_thread_list;
extern unsigned long pkts_per_run_array[];

#endif
