#include <asm/uaccess.h>

#include <linux/audit.h>
#include <linux/audit_event.h>

#ifdef CONFIG_ACL
#include <linux/acl.h>
#endif

int audit_acct(int status, const char *filename)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_ACCT)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}
	
	audit_resolve_filename(data->file, &data->file_len, filename,
	                       STRLEN_USER(filename));

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_ACCT, status)) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);
	audit_fill_cfiledata(data, filename);

	rec->event = AUE_ACCT;

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = 0;
	data->arg2 = 0;

	data->env = NULL;

	data->file2_len = 0;

	rec->state = AU_FULL;
	
	return 0;
}
