#include <linux/fs.h>

#include <linux/audit.h>
#include <linux/audit_event.h>

int audit_ioperm(int status, unsigned long from, unsigned long num, int turn_on)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_IOPERM)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_fill_static(data);
	audit_fill_filedata(data, NULL);

	rec->event = AUE_IOPERM;

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = (unsigned short) from;
	data->arg1 += (unsigned short) num << 16;

	data->arg2 = turn_on;

	data->env = NULL;

	data->file_len = 0;
	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


int audit_iopl(int status, unsigned long unused)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_IOPL)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_fill_static(data);
	audit_fill_filedata(data, NULL);

	rec->event = AUE_IOPL;

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = unused;
	data->arg2 = 0;

	data->env = NULL;

	data->file_len = 0;
	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}
