#include <asm/uaccess.h>

#include <linux/fs.h>
#include <linux/module.h>

#include <linux/audit.h>
#include <linux/audit_event.h>

int audit_init_module(int status, const char *name, struct module *mod)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_INIT_MODULE)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_fill_static(data);
	audit_fill_cfiledata(data, name);

	rec->event = AUE_INIT_MODULE;

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = 0;
	data->arg2 = 0;

	data->env = 0;

	if ((data->file_len = STRLEN_USER(name)) > 0) {
		if (copy_from_user(data->file, name, data->file_len)) {
			data->file_len = 0;
		} else {
			if (data->file[data->file_len - 1] == '\0')
				--data->file_len;
		}
	}

	data->file2_len = 0;
	if (rec) {
		int len;

		if ((len = STRLEN_USER(mod->name)) <= 0)
			goto out;

		if (copy_from_user(data->file2, mod->name, len))
			goto out;

		data->file2_len = len;
	}

out:
	rec->state = AU_FULL;

	return 0;
}


int audit_delete_module(int status, const char * name)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	rec = audit_get_free_record(AU_HOST_REC, 1, AUE_DELETE_MODULE);
	if (!rec)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_fill_static(data);
	audit_fill_cfiledata(data, name);

	rec->event = AUE_DELETE_MODULE;

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = 0;
	data->arg2 = 0;

	data->env = NULL;

	if ((data->file_len = STRLEN_USER(name)) > 0) {
		if (copy_from_user(data->file, name, data->file_len)) {
			data->file_len = 0;
		} else {
			if (data->file[data->file_len - 1] == '\0')
				--data->file_len;
		}
	}

	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}
