#include <asm/uaccess.h>

#include <linux/string.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/sched.h>

#include <linux/audit.h>
#include <linux/audit_event.h>

#ifdef CONFIG_ACL
#include <linux/acl.h>
#endif

int audit_link(unsigned long * args)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_LINK)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, (char *) args[1],
	                       (int) args[2]);

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_LINK, (int) args[0])) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);
	audit_fill_cfiledata(data, (char *) args[1]);

	if (!(int) args[0])
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = 0;
	data->arg2 = 0;

	data->env = NULL;

	audit_resolve_filename(data->file2, &data->file2_len, (char *) args[3],
	                       (int) args[4]);

	rec->state = AU_FULL;
	
	return 0;
}


int audit_unlink(int status, const char * filename, struct inode *ifile)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_UNLINK)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, filename,
	                       STRLEN_USER(filename));

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_UNLINK, status)) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	if (ifile) {
		audit_fill_filedata(data, ifile);
	} else {
		if (!status) {
			audit_fill_filedata(data, NULL);
		} else {
			audit_fill_cfiledata(data, filename);
		}
	}

	data->arg1 = 0;
	data->arg2 = 0;

	data->env = NULL;

	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


int audit_mknod(unsigned long * args)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_MKNOD)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, (char *) args[1],
	                       STRLEN_USER((char *) args[1]));
	
#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_MKNOD, (int) args[0])) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);
	audit_fill_cfiledata(data, (char *) args[1]);

	if (!(int) args[0])
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = args[2];
	data->arg2 = args[3];

	data->env = NULL;

	data->file2_len = 0;

	rec->state = AU_FULL;
	
	return 0;
}


int audit_rename(unsigned long * args)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_RENAME)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, (char *) args[1],
	                       (int) args[2]);

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_RENAME, (int) args[0])) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);

	if (!(int) args[0]) {
		data->status = AU_SYSCALL_SUCCESSFULL;

		audit_fill_cfiledata(data, (char *) args[3]);
	} else {
		data->status = AU_SYSCALL_FAILED;

		audit_fill_cfiledata(data, (char *) args[1]);
	}

	data->arg1 = 0;
	data->arg2 = 0;

	data->env = NULL;

	audit_resolve_filename(data->file2, &data->file2_len, (char *) args[3],
	                       (int) args[4]);

	rec->state = AU_FULL;

	return 0;
}


int audit_mkdir(unsigned long * args)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_MKDIR)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, (char *) args[1],
	                       (int) args[2]);

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_MKDIR, (int) args[0])) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);
	audit_fill_cfiledata(data, (char *) args[1]);

	if (!(int) args[0])
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = args[3];
	data->arg2 = 0;

	data->env = NULL;

	data->file2_len = 0;

	rec->state = AU_FULL;
	
	return 0;
}


int audit_rmdir(int status, const char * filename, struct inode *ifile)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_RMDIR)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, filename,
	                       STRLEN_USER(filename));

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_MKDIR, status)) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	if (ifile) {
		audit_fill_filedata(data, ifile);
	} else {
		if (!status) {
			audit_fill_filedata(data, NULL);
		} else {
			audit_fill_cfiledata(data, filename);
		}
	}

	data->arg1 = 0;
	data->arg2 = 0;

	data->env = NULL;

	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


int audit_symlink(unsigned long * args)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_SYMLINK)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, (char *) args[1],
	                       (int) args[2]);

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                    AUE_SYMLINK, (int) args[0])) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);
	audit_fill_cfiledata(data, (char *) args[1]);

	if (!(int) args[0])
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = 0;
	data->arg2 = 0;

	data->env = NULL;

	audit_resolve_filename(data->file2, &data->file2_len, (char *) args[3],
	                       (int) args[4]);

	rec->state = AU_FULL;
	
	return 0;
}
