#include <asm/uaccess.h>
#include <asm/unistd.h>
#include <asm/fcntl.h>

#include <linux/string.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/file.h>
#include <linux/sched.h>
#include <linux/utime.h>
#include <linux/highuid.h>

#include <linux/audit.h>
#include <linux/audit_event.h>

#include "buffer.h"

#ifdef CONFIG_ACL
#include <linux/acl.h>
#endif

int audit_open(unsigned long *args)
{
	int event;
	struct audit_record *rec;
	struct audit_host_record *data;

	event = AUE_OPEN_R;
	
	if ((int) args[5]) {
		if ((int) args[5] == 1)
			event = AUE_USELIB;
		else
			event = AUE_CREAT;

		goto check_event_out;
	}

	if ((int) args[3] & O_WRONLY) {
		if ((int) args[3] & O_CREAT) {
			if ((int) args[3] & O_TRUNC) {
				event = AUE_OPEN_WTC;
				goto check_event_out;
			}

			event = AUE_OPEN_WC;
			goto check_event_out;
		}

		if ((int) args[3] & O_TRUNC) {
			event = AUE_OPEN_WT;
			goto check_event_out;
		}

		event = AUE_OPEN_W;
		goto check_event_out;
	}

	if ((int) args[3] & O_RDWR) {
		if ((int) args[3] & O_CREAT) {
			if ((int) args[3] & O_TRUNC) {
				event = AUE_OPEN_RWTC;
				goto check_event_out;
			}

			event = AUE_OPEN_RWC;
			goto check_event_out;
		}

		if ((int) args[3] & O_TRUNC) {
			event = AUE_OPEN_RWT;
			goto check_event_out;
		}

		event = AUE_OPEN_RW;
		goto check_event_out;
	}

	if ((int) args[3] & O_CREAT) {
		if ((int) args[3] & O_TRUNC) {
			event = AUE_OPEN_RTC;
			goto check_event_out;
		}

		event = AUE_OPEN_RC;
		goto check_event_out;
	}

	if ((int) args[3] & O_TRUNC)
		event = AUE_OPEN_RT;

check_event_out:
	if (!audit_doit(event, (int) args[0]))
		return 0;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, event)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, (char *) args[1],
	                       (int) args[2]);

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16), event,
		                   (int) args[0])) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);

	if ((int) args[0] < 0)
		data->status = AU_SYSCALL_FAILED;
	else
		data->status = AU_SYSCALL_SUCCESSFULL;

	audit_fill_cfiledata(data, (char *) args[1]);

	data->arg1 = args[4];
	data->arg2 = 0;

	data->env = NULL;

	data->file2_len = 0;

	rec->state = AU_FULL;
	
	return 0;
}


int audit_chdir(int status, const char *filename)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_CHDIR)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_fill_static(data);

	if (!status) {
		data->status = AU_SYSCALL_SUCCESSFULL;

		if (*filename != '/') {
			audit_resolve_filename(data->file, &data->file_len, "./", 2);
		} else {
			audit_resolve_filename(data->file, &data->file_len, filename,
			                       STRLEN_USER(filename));
		}

		audit_fill_filedata(data, current->fs->pwd->d_inode);
	} else {
		data->status = AU_SYSCALL_FAILED;

		audit_resolve_filename(data->file, &data->file_len, filename,
		                       STRLEN_USER(filename));
		audit_fill_cfiledata(data, data->file);
	}

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_CHDIR, status)) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	data->arg1 = 0;
	data->arg2 = 0;

	data->env = NULL;

	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


int audit_chmod(unsigned long * args)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_CHMOD)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, (char *) args[1],
	                       STRLEN_USER((char *) args[1]));

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_CHMOD, (int) args[0])) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);
	audit_fill_cfiledata(data, (char *) args[1]);

	if (!(int) args[0]) {
		data->status = AU_SYSCALL_SUCCESSFULL;
		data->perm = (unsigned long) args[3];
	} else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = (mode_t) args[2];
	data->arg2 = 0;

	data->env = NULL;

	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


static int generic_chown(struct audit_host_record *data, int event, int error,
                         uid_t user, gid_t group, uid_t i_uid, gid_t i_gid)
{
#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   event, error))
			return AU_UNUSED;
	}
#endif

	audit_fill_static(data);

	if (!error) {
		data->status = AU_SYSCALL_SUCCESSFULL;

		data->owner = i_uid;
		data->gowner = i_gid;
	} else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = user;
	data->arg2 = group;

	data->env = NULL;

	data->file2_len = 0;

	return AU_FULL;
}

static int __audit_chown(int error, const char *filename, uid_t user,
                         gid_t group, uid_t i_uid, gid_t i_gid)
{
	struct audit_record *rec;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_CHOWN)) != NULL) {
		struct audit_host_record *data;
	
		data = (struct audit_host_record *) rec->data;
		if (data) {
			audit_resolve_filename(data->file, &data->file_len, filename,
			                       STRLEN_USER(filename));
			audit_fill_cfiledata(data, filename);

			rec->state = generic_chown(data, AUE_CHOWN, error, user, group,
			                           i_uid, i_gid);
		} else
			rec->state = AU_UNUSED;
	}

	return 0;
}

asmlinkage long audit_chown(const char *filename, uid_t user, gid_t group)
{
	struct nameidata nd;
	long error;
	uid_t i_uid = 0;
	gid_t i_gid = 0;

	error = (long) user_path_walk(filename, &nd);
	if (!error) {
		if (nd.dentry->d_inode) {
			i_uid = nd.dentry->d_inode->i_uid;
			i_gid = nd.dentry->d_inode->i_gid;
		}

		path_release(&nd);
	}

	error = ((long (*) (const char *, uid_t, gid_t)) orig_sys_call_table[__NR_chown32])(filename, user, group);
	if (audit_doit(AUE_CHOWN, (int) error))
		__audit_chown((int) error, filename, user, group, i_uid, i_gid);

	return error;
}

asmlinkage long audit_chown16(const char *filename, old_uid_t user,
                              old_gid_t group)
{
	struct nameidata nd;
	long error;
	uid_t i_uid = 0;
	gid_t i_gid = 0;

	error = (long) user_path_walk(filename, &nd);
	if (!error) {
		if (nd.dentry->d_inode) {
			i_uid = nd.dentry->d_inode->i_uid;
			i_gid = nd.dentry->d_inode->i_gid;
		}

		path_release(&nd);
	}

	error = ((long (*) (const char *, old_uid_t, old_gid_t)) orig_sys_call_table[__NR_chown])(filename, user, group);
	if (audit_doit(AUE_CHOWN, (int) error))
		__audit_chown((int) error, filename, low2highuid(user),
		              low2highgid(group), i_uid, i_gid);

	return error;
}


static inline int __audit_lchown(int error, const char *filename, uid_t user,
                                 gid_t group, uid_t i_uid, gid_t i_gid)
{
	struct audit_record *rec;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_LCHOWN)) != NULL) {
		struct audit_host_record *data;
	
		data = (struct audit_host_record *) rec->data;
		if (data) {
			audit_resolve_filename(data->file, &data->file_len, filename,
			                       STRLEN_USER(filename));
			audit_fill_cfiledata(data, filename);

			rec->state = generic_chown(data, AUE_LCHOWN, error, user, group,
			                           i_uid, i_gid);
		} else
			rec->state = AU_UNUSED;
	}

	return 0;
}

asmlinkage long audit_lchown(const char *filename, uid_t user, gid_t group)
{
	struct nameidata nd;
	long error;
	uid_t i_uid = 0;
	gid_t i_gid = 0;

	error = (long) user_path_walk(filename, &nd);
	if (!error) {
		if (nd.dentry->d_inode) {
			i_uid = nd.dentry->d_inode->i_uid;
			i_gid = nd.dentry->d_inode->i_gid;
		}

		path_release(&nd);
	}

	error = ((long (*) (const char *, uid_t, gid_t)) orig_sys_call_table[__NR_lchown32])(filename, user, group);
	if (audit_doit(AUE_LCHOWN, (int) error))
		__audit_lchown((int) error, filename, user, group, i_uid, i_gid);

	return error;
}

asmlinkage long audit_lchown16(const char * filename, old_uid_t user,
                               old_gid_t group)
{
	struct nameidata nd;
	long error;
	uid_t i_uid = 0;
	gid_t i_gid = 0;

	error = (long) user_path_walk(filename, &nd);
	if (!error) {
		if (nd.dentry->d_inode) {
			i_uid = nd.dentry->d_inode->i_uid;
			i_gid = nd.dentry->d_inode->i_gid;
		}

		path_release(&nd);
	}

	error = ((long (*) (const char *, old_uid_t, old_gid_t)) orig_sys_call_table[__NR_lchown])(filename, user, group);
	if (audit_doit(AUE_LCHOWN, (int) error))
		__audit_lchown((int) error, filename, low2highuid(user),
		               low2highgid(group), i_uid, i_gid);

	return error;
}


static int __audit_fchown(int error, struct file *f, uid_t user, gid_t group,
                          uid_t i_uid, gid_t i_gid)
{
	struct audit_record *rec;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_FCHOWN)) != NULL) {
		struct audit_host_record *data;

		data = (struct audit_host_record *) rec->data;
		if (data) {
			if (f && f->f_dentry) {
				data->file_len = audit_resolve_dentry(data->file, f->f_dentry,
				                                      f->f_vfsmnt);
				audit_fill_filedata(data, f->f_dentry->d_inode);
			} else {
				audit_fill_filedata(data, NULL);
				data->file_len = 0;
			}

			rec->state = generic_chown(data, AUE_FCHOWN, error, user, group,
			                           i_uid, i_gid);
		} else
			rec->state = AU_UNUSED;
	}

	return 0;
}

asmlinkage long audit_fchown(unsigned int fd, uid_t user, gid_t group)
{
	struct file *file;
	long error;
	uid_t i_uid = 0;
	gid_t i_gid = 0;

	file = fget(fd);
	if (file && file->f_dentry && file->f_dentry->d_inode) {
		i_uid = file->f_dentry->d_inode->i_uid;
		i_gid = file->f_dentry->d_inode->i_gid;
	}

	error = ((long (*) (unsigned int, uid_t, gid_t)) orig_sys_call_table[__NR_fchown32])(fd, user, group);
	if (audit_doit(AUE_FCHOWN, (int) error))
		__audit_fchown((int) error, file, user, group, i_uid, i_gid);

	if (file)
		fput(file);

	return error;
}

asmlinkage long audit_fchown16(unsigned int fd, old_uid_t user, old_gid_t group)
{
	struct file *file;
	long error;
	uid_t i_uid = 0;
	gid_t i_gid = 0;

	file = fget(fd);
	if (file && file->f_dentry && file->f_dentry->d_inode) {
		i_uid = file->f_dentry->d_inode->i_uid;
		i_gid = file->f_dentry->d_inode->i_gid;
	}

	error = ((long (*) (unsigned int, old_uid_t, old_gid_t)) orig_sys_call_table[__NR_fchown])(fd, user, group);
	if (audit_doit(AUE_FCHOWN, (int) error))
		__audit_fchown((int) error, file, low2highuid(user), low2highgid(group),
		               i_uid, i_gid);

	if (file)
		fput(file);

	return error;
}


int audit_utime(int status, const char * filename, struct utimbuf * times)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_UTIME)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, filename,
	                       STRLEN_USER(filename));

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_UTIME, status)) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);
	audit_fill_cfiledata(data, filename);

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	if (times) {
		struct utimbuf new_times;

		if (!copy_from_user(&new_times, times, sizeof(*times))) {
			data->arg1 = new_times.actime;
			data->arg2 = new_times.modtime;
		} else {
			data->arg1 = 0;
			data->arg2 = 0;
		}
	} else {
		data->arg1 = CURRENT_TIME;
		data->arg2 = CURRENT_TIME;
	}

	data->env = NULL;

	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


int audit_utimes(int status, const char * filename, struct timeval * utimes)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_UTIMES)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, filename,
	                       STRLEN_USER(filename));

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_UTIMES, status)) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif

	audit_fill_static(data);
	audit_fill_cfiledata(data, filename);

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	if (utimes) {
		struct timeval times[2];

		if (!copy_from_user(&times, utimes, sizeof(times))) {
			data->arg1 = times[0].tv_sec;
			data->arg2 = times[1].tv_sec;
		} else {
			data->arg1 = 0;
			data->arg2 = 0;
		}
	} else {
		data->arg1 = CURRENT_TIME;
		data->arg2 = CURRENT_TIME;
	}

	data->env = NULL;
	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


struct audit_record *audit_chroot_pre(const char *filename)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_CHROOT)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_fill_static(data);
	audit_fill_cfiledata(data, filename);

	data->arg1 = 0;
	data->arg2 = 0;

	data->env = NULL;

	audit_resolve_filename(data->file, &data->file_len, filename,
	                       STRLEN_USER(filename));

	data->file2_len = 0;

	return rec;
}


int audit_chroot_post(struct audit_record *rec, int status)
{
	struct audit_host_record *data;

	if (!rec)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if ((atomic_read(&au_svc.state) != AU_DOIT) || !data) {
		rec->state = AU_UNUSED;
		return 0;
	}

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_CHROOT, status)) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif
	
	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	rec->state = AU_FULL;

	return 0;
}


int audit_truncate(int status, const char * filename, unsigned long len)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_TRUNCATE)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_resolve_filename(data->file, &data->file_len, filename,
	                       STRLEN_USER(filename));

#ifdef CONFIG_ACL
	if (data->file_len) {
		if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
		                   AUE_TRUNCATE, status)) {
			rec->state = AU_UNUSED;
			return 0;
		}
	}
#endif
	
	audit_fill_static(data);
	audit_fill_cfiledata(data, filename);

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = len;
	data->arg2 = 0;

	data->env = NULL;

	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


int audit_ftruncate(int status, unsigned int fd, unsigned long len)
{
	struct audit_record *rec;
	struct audit_host_record *data;
	struct file *f = NULL;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_FTRUNCATE)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	f = fget(fd);

	if (f) {
		data->file_len = audit_resolve_dentry(data->file, f->f_dentry,
		                                      f->f_vfsmnt);

		if (data->file_len) {
#ifdef CONFIG_ACL
			if (data->file_len) {
				if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
				                   AUE_FTRUNCATE, status)) {
					rec->state = AU_UNUSED;
					fput(f);

					return 0;
				}
			}
#endif

			audit_fill_filedata(data, f->f_dentry->d_inode);
		}

		fput(f);
	} else {
		audit_fill_filedata(data, NULL);
		data->file_len = 0;
	}

	audit_fill_static(data);

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = len;
	data->arg2 = 0;
	data->env = NULL;
	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


int audit_fchmod(unsigned long * args)
{
	struct audit_record *rec;
	struct audit_host_record *data;
	struct file *f = NULL;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_FCHMOD)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	f = fget((unsigned int) args[1]);

	if (f && f->f_dentry) {
		data->file_len = audit_resolve_dentry(data->file, f->f_dentry,
		                                      f->f_vfsmnt);

#ifdef CONFIG_ACL
		if (data->file_len) {
			if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
			                   AUE_FCHMOD, (int) args[0])) {
				rec->state = AU_UNUSED;
				fput(f);

				return 0;
			}
#endif
		}

		audit_fill_filedata(data, f->f_dentry->d_inode);

		fput(f);
	} else {
		audit_fill_filedata(data, NULL);
		data->file_len = 0;
	}

	audit_fill_static(data);

	if (!(int) args[0]) {
		data->status = AU_SYSCALL_SUCCESSFULL;
		data->perm = (unsigned long) args[3];
	} else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = args[2];
	data->arg2 = 0;
	data->env = NULL;
	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


int audit_fchdir(int status, unsigned int fd)
{
	struct audit_record *rec;
	struct audit_host_record *data;
	struct file *f = NULL;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_FCHDIR)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	f = fget(fd);

	if (f && f->f_dentry) {
		data->file_len = audit_resolve_dentry(data->file, f->f_dentry,
		                                      f->f_vfsmnt);

#ifdef CONFIG_ACL
		if (data->file_len) {
			if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
			                   AUE_FCHDIR, status)) {
				rec->state = AU_UNUSED;
				fput(f);

				return 0;
			}
		}
#endif

		audit_fill_filedata(data, f->f_dentry->d_inode);

		fput(f);
	} else {
		audit_fill_filedata(data, NULL);
		data->file_len = 0;
	}

	audit_fill_static(data);

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = 0;
	data->arg2 = 0;
	data->env = NULL;
	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}

int audit_mmap_exec(int status, unsigned long fd, unsigned long prot,
                    unsigned long flags)
{
	struct audit_record *rec;
	struct audit_host_record *data;
	struct file *f = NULL;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_MMAP_EXEC)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	f = fget(fd);
	if (f && f->f_dentry) {
		data->file_len = audit_resolve_dentry(data->file, f->f_dentry,
		                                      f->f_vfsmnt);

#ifdef CONFIG_ACL
		if (data->file_len) {
			if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
			                   AUE_FCHDIR, status)) {
				rec->state = AU_UNUSED;
				fput(f);

				return 0;
			}
		}
#endif

		audit_fill_filedata(data, f->f_dentry->d_inode);
		fput(f);
	} else {
		audit_fill_filedata(data, NULL);
		data->file_len = 0;
	}

	audit_fill_static(data);
	if (status < 0)
		data->status = AU_SYSCALL_FAILED;
	else
		data->status = AU_SYSCALL_SUCCESSFULL;
		

	data->arg1 = prot;
	data->arg2 = flags;
	data->env = NULL;
	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}
