#include <asm/unistd.h>
#include <linux/fs.h>

#include <linux/audit.h>
#include <linux/audit_event.h>

#include "buffer.h"

static int __audit_ptrace(int status, pid_t pid, long request)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_VTRACE)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_fill_static(data);
	audit_fill_filedata(data, NULL);

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = pid;
	data->arg2 = request;
	data->env = NULL;
	data->file_len = 0;
	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}

asmlinkage int audit_ptrace(long request, long pid, long addr, long data)
{
	int error;

	error = -EPERM;
	if (!AU_SVC_INITIALIZED(&au_svc) || (pid != audit_get_pid()))
		error = ((int (*) (long, long, long, long)) orig_sys_call_table[__NR_ptrace])(request, pid, addr, data);

	__audit_ptrace(error, pid, request);

	return error;
}


