#include <asm/uaccess.h>

#include <linux/fs.h>

#include <linux/audit.h>
#include <linux/audit_event.h>

#ifdef CONFIG_ACL
#include <linux/acl.h>
#endif

#define AU_FS_HASH 77
#define AU_FS_HASH_FUNC(x, c, h) ((64 * x + (int) c) % h)

#define FS_TYPES "code", "affs", "adfs", "minix", "xiafs", "ext", "ext2", \
	"msdos", "umsdos", "vfat", "proc", "autofs", "devpts", "nfs", \
	"iso9660", "smbfs", "ncpfs", "hfs", "hpfs", "ntfs", "qnx4", "romfs", \
	"ufs", "sysv", "xenix", "coherent"

#define FS_TYPES_NO 26

int audit_mount(unsigned long *args)
{
	char *q = NULL;
	int no_dev_param = 0;
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_MOUNT)) == NULL)
		return 0;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	if ((q = (char *) args[4]) != NULL) {
		int i;
		char *p = NULL, *fs_type[] = {FS_TYPES};
		unsigned long type = 0;

		if (copy_from_user(&type, q, sizeof(unsigned long)) < 0)
			goto unknown_type;

		for (i = 0; i < FS_TYPES_NO; i++) {
			if (*(const unsigned long *) &type == *(const unsigned long *) "nfs") {
				no_dev_param = 1;
				p = fs_type[i];

				break;
			}

			if (*(const unsigned long *) &type == *(const unsigned long *) fs_type[i])
				p = fs_type[i];
		}

		if (!p)
			goto unknown_type;

		for (i = 0; *p != '\0'; p++)
			i = AU_FS_HASH_FUNC(i, *p, AU_FS_HASH);

		data->arg2 = i;
	} else {
unknown_type:
		data->arg2 = 0;
	}


	if (no_dev_param) {
		audit_fill_cfiledata(data, NULL);

		if ((data->file_len = STRLEN_USER(q)) > 0) {
			if (copy_from_user(data->file, q, data->file_len) < 0)
				data->file_len = 0;
			else {
				if (data->file[data->file_len - 1] == '\0')
					data->file_len--;
			}
		}
	} else {
		audit_resolve_filename(data->file, &data->file_len, (char *) args[1],
		                       STRLEN_USER((char *) args[1]));
		
#ifdef CONFIG_ACL
		if (data->file_len) {
			if (acl_check_host(data->file + (__u16) (data->file_len >> 16),
			                   AUE_MOUNT, (int) args[0])) {
				rec->state = AU_UNUSED;
				return 0;
			}
		}
#endif

		audit_fill_cfiledata(data, (char *) args[1]);
	}

	audit_fill_static(data);

	if (!(int) args[0])
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	data->arg1 = args[3];
	data->env = NULL;

	audit_resolve_filename(data->file2, &data->file2_len, (char *) args[2],
		STRLEN_USER((char *) args[2]));


	rec->state = AU_FULL;

	return 0;
}
