#include <linux/if_arp.h>

#include <linux/audit.h>
#include <linux/audit_event.h>

extern struct audit_dyn_buf *au_net_env_buf;

#if defined(CONFIG_IPV6) || defined (CONFIG_IPV6_MODULE)

#include <linux/ipv6.h>

int audit_ipv6_rcv(struct sk_buff *skb, unsigned short sub_evt, short exthdr)
{
	struct audit_record *rec;

	rec = audit_get_free_record(AU_NET_REC, 0, AUE_IPV6_RCV);
	if (rec) {
		struct audit_net_record *data;

		data = (struct audit_net_record *) rec->data;
		if (data) {
			rec->pid = sub_evt;
			rec->pid |= exthdr << 16;

			audit_copy_dev(data, skb->dev, IFNAMSIZ);
			audit_copy_mac(data, skb, skb->dev);
			audit_copy_ipv6(data, skb->nh.ipv6h, skb->nh.ipv6h->nexthdr);

			data->h_prot_size = 0;
		
			data->env = NULL;

			rec->state = AU_FULL;

			return 0;
		}

		rec->state = AU_UNUSED;
	}
	
	return -EAGAIN;
}

#endif

int audit_arp_rcv(struct sk_buff *skb, unsigned int sub_evt)
{
	struct audit_record *rec;
	struct audit_net_record *data;

	if ((rec = audit_get_free_record(AU_NET_REC, 0, AUE_ARP_RCV)) == NULL)
		return 0;

	data = (struct audit_net_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}
	
	rec->pid = sub_evt;

	audit_copy_dev(data, skb->dev, IFNAMSIZ);
	audit_copy_mac(data, skb, skb->dev);

	memcpy(&data->nh.arph, skb->nh.arph, sizeof(struct arphdr));
	data->nh_prot_size = sizeof(struct arphdr);

	data->h_prot_size = 0;

	if (au_net_env_buf) {
		void *src[5];

		unsigned long sha[2];
		unsigned long sip[2];
		unsigned long tha[2];
		unsigned long tip[2];

		unsigned char *arp_ptr = (unsigned char *)(skb->nh.arph + 1);

		sha[0] = (unsigned long) skb->nh.arph->ar_hln;
		sha[1] = (unsigned long) arp_ptr;
		src[0] = (void *) sha;

		arp_ptr += skb->nh.arph->ar_hln;
		sip[0] = (unsigned long) skb->nh.arph->ar_pln;
		sip[1] = (unsigned long) arp_ptr;
		src[1] = (void *) sip;

		arp_ptr += skb->nh.arph->ar_pln;
		tha[0] = (unsigned long) skb->nh.arph->ar_hln;
		tha[1] = (unsigned long) arp_ptr;
		src[2] = (void *) tha;

		arp_ptr += skb->nh.arph->ar_hln;
		tip[0] = (unsigned long) skb->nh.arph->ar_pln;
		tip[1] = (unsigned long) arp_ptr;
		src[3] = (void *) tip;

		src[4] = NULL;

		audit_fill_dyn_buf2(au_net_env_buf, &data->env, src, 4, AU_K_SPACE);
	} else 
		data->env = NULL;

	rec->state = AU_FULL;
	
	return 0;
}


int audit_ipv4_rcv(struct sk_buff *skb, unsigned int sub_evt)
{
	struct audit_record *rec;
	struct audit_net_record *data;

	if ((rec = audit_get_free_record(AU_NET_REC, 0, AUE_IP_RCV)) == NULL)
		return 0;

	data = (struct audit_net_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}
	
	rec->pid = sub_evt;

	audit_copy_dev(data, skb->dev, IFNAMSIZ);
	audit_copy_mac(data, skb, skb->dev);
	audit_copy_ipv4(data, skb->nh.iph);

	data->h_prot_size = 0;

	data->env = NULL;

	rec->state = AU_FULL;
	
	return 0;
}

