#include <asm/uaccess.h>

#include <linux/timex.h>
#include <linux/mm.h>

#include <linux/audit.h>
#include <linux/audit_event.h>


int audit_stime(int status, int *tptr)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_STIME)) == NULL)
		return -EFAULT;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_fill_static(data);
	audit_fill_filedata(data, NULL);

	if (!status)
		data->status = AU_SYSCALL_SUCCESSFULL;
	else
		data->status = AU_SYSCALL_FAILED;

	if (get_user(data->arg1, tptr))
		data->arg1 = 0;

	data->arg2 = 0;

	data->env = NULL;
	data->file_len = 0;
	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}


int audit_settimeofday(int status, struct timeval *time)
{
	struct audit_record *rec;
	int ret;

	ret = -EFAULT;
	rec = audit_get_free_record(AU_HOST_REC, 1, AUE_SETTIMEOFDAY);
	if (rec) {
		struct audit_host_record *data;

		ret = -EAGAIN;
		data = (struct audit_host_record *) rec->data;
		if (data) {
			audit_fill_static(data);
			audit_fill_filedata(data, NULL);

			data->status = status ? AU_SYSCALL_FAILED : AU_SYSCALL_SUCCESSFULL;

			data->arg1 = 0;
			data->arg2 = 0;
			if (time) {
				struct timeval new_time;

				if (!copy_from_user(&new_time, time, sizeof(*time))) {
					data->arg1 = new_time.tv_sec;
					data->arg2 = new_time.tv_usec;
				}
			}

			data->env = NULL;
			data->file_len = 0;
			data->file2_len = 0;

			rec->state = AU_FULL;
		} else
			rec->state = AU_UNUSED;
	}

	return ret;
}


int audit_adjtimex(int status)
{
	struct audit_record *rec;
	struct audit_host_record *data;

	if ((rec = audit_get_free_record(AU_HOST_REC, 1, AUE_ADJTIME)) == NULL)
		return -EFAULT;

	data = (struct audit_host_record *) rec->data;
	if (!data) {
		rec->state = AU_UNUSED;
		return 0;
	}

	audit_fill_static(data);
	audit_fill_filedata(data, NULL);

	rec->event = AUE_ADJTIME;

	if (status < 0)
		data->status = AU_SYSCALL_FAILED;
	else
		data->status = AU_SYSCALL_SUCCESSFULL;

	data->arg1 = 0;
	data->arg2 = 0;
	data->env = NULL;
	data->file_len = 0;
	data->file2_len = 0;

	rec->state = AU_FULL;

	return 0;
}
