#include <linux/init.h>
#include <linux/sched.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/string.h>
#include <linux/slab.h>

#include <asm/uaccess.h>

#include <linux/audit.h>

#include "buffer.h"

static int au_root = 0;
static int au_modus = AU_NORMAL;
static struct audit_cap_db *au_cap_db = NULL;

int audit_getadmin(void)
{
	return au_root;
}


int audit_getmodus(void)
{
	return au_modus;
}

int audit_check_capability(int cap)
{
	struct audit_cap_db *p;

	if ((au_modus != AU_SECURE) || (current->audit->aid == au_root) ||
	    (current->audit->set == 0))
		return 1;

	for (p = au_cap_db; p; p = p->next) {
		if (p->cap == cap && p->aid == current->audit->aid)
			return 1;
	}

	if (cap == AU_CAP_MSG) {
		p = au_cap_db;

		while (p) {
			if (p->cap == AU_CAP_MSG) {
				if (p->prog_id.inode == current->audit->comm_ino) {
					if (p->prog_id.dev == current->audit->comm_dev &&
					    !strncmp(p->prog_id.comm, current->comm, 15)) {
					
						return 1;
					}
				}
			}

			p = p->next;
		}
	}

	return 0;
}

static int audit_set_prog_cap(int cap, char *prog, unsigned long inode, kdev_t dev)
{
	struct audit_cap_db *p = au_cap_db, *q = NULL;
	int i, j, ret;
	char ch;

	if (cap < AU_CAP_ADM || cap > AU_CAP_MSG)
		return -EINVAL;

	if (!prog)
		return -EFAULT;

	while (p) {
		q = p;

		if (p->prog_id.inode == inode && p->prog_id.dev == dev) {
			ret = 0;
			for (i = 0, j = 0; i < 15 && (ch = prog[j]) != '\0'; i++, j++) {
				if (ch == '/') {
					i = ret = 0;
					continue;
				}

				if (prog[j] != p->prog_id.comm[i])
					ret = 1;
			}

			if (!ret)
				return 0;
		}

		p = p->next;
	}

	p = (struct audit_cap_db *) kmalloc(sizeof(*p), GFP_KERNEL);

	if (!p)
		return -ENOMEM;
	
	p->cap = cap;
	p->aid = au_root;

	for (i=0; (ch = *(prog++)) != '\0';) {
		if (ch == '/')
			i = 0;
		else
			if (i < 15)
				p->prog_id.comm[i++] = ch;
	}

	for ( ; i < 16; i++)
		p->prog_id.comm[i] = '\0';

	p->prog_id.inode = inode;
	p->prog_id.dev = dev;

	p->next = NULL;

	if (!q)
		au_cap_db = p;
	else
		q->next = p;

	return 0;
}

static int audit_set_capability(int cap, int aid)
{
	struct audit_cap_db *p = au_cap_db, *q = NULL;

	if (cap < AU_CAP_ADM || cap > AU_CAP_MSG)
		return -EINVAL;

	if (aid == au_root)
		return 0;
	
	while (p) {
		q = p;

		if (p->cap == cap && p->aid == aid)
			return 0;

		p = p->next;
	}

	p = (struct audit_cap_db *) kmalloc(sizeof(*p), GFP_KERNEL);

	if (!p)
		return -ENOMEM;
	
	p->cap = cap;
	p->aid = aid;

	memset(p->prog_id.comm, '\0', 16);
	p->prog_id.inode = 0;
	p->prog_id.dev = 0;

	p->next = NULL;

	if (!q)
		au_cap_db = p;
	else
		q->next = p;

	return 0;
}

static int audit_get_capability(int aid, int *cap)
{
	int ret;
	struct audit_cap_db *p = au_cap_db;

	ret = -ESRCH;
	while (p) {
		if (p->aid == aid) {
			ret = 0;
			switch (p->cap) {
				case AU_CAP_ADM : {
					cap[0] = 1;
					break;
				}
				case AU_CAP_SETID : {
					cap[1] = 1;
					break;
				}
				case AU_CAP_MSG : {
					cap[2] = 1;
					break;
				}
			}
		}

		p = p->next;
	}

	return ret;
}


int audit_setadmin(uid_t admin)
{
	static int busy = 0;

	if (busy)
		return -EPERM;
	
	au_root = admin;
	busy++;
	return 0;
}


int audit_setmod(int mod)
{
	if (au_modus == AU_SECURE)
		return -EPERM;

	if (mod < AU_DISABLED || mod > AU_SECURE)
		return -EINVAL;

	if (mod == AU_DISABLED && atomic_read(&au_svc.state) == AU_DOIT)
		return -EBUSY;

	if (mod == AU_SECURE && AU_SVC_INITIALIZED(&au_svc)) {
		int auditd_aid;

		auditd_aid = au_svc.auditd_task->audit->aid;
		if (auditd_aid != audit_getadmin())
			return -EPERM;
	}

	au_modus = mod;
	return 0;
}

int sys_audit_cap_ctl(int cmd, unsigned long *args)
{
	int ret;

	if (au_modus == AU_DISABLED && cmd != AU_CAP_GETMOD && cmd != AU_CAP_CHMOD)
		return -ESRCH;

	if (!audit_check_capability(AU_CAP_ADM))
		return -EPERM;

	if (cmd < AU_CAP_SET || cmd > AU_CAP_GETMOD)
		return -EINVAL;

	ret = 0;
	switch (cmd) {
		case AU_CAP_SET : {
			unsigned long a[2];

			if (au_modus == AU_SECURE)
				return -EPERM;

			if (copy_from_user(&a, args, 2 * sizeof(unsigned long)))
				return -EFAULT;
	
			switch ((int) a[0]) {
				case AU_CAP_SET_AID : {
					int aid_a[2];

					ret = -EFAULT;
					if (!copy_from_user(&aid_a, (int *) a[1], 2 * sizeof(int)))
						ret = audit_set_capability(aid_a[0], aid_a[1]);

					break;
				}
				case AU_CAP_SET_PROG : {
					unsigned long prog_a[4];

					ret = -EFAULT;
					if (!copy_from_user(&prog_a, (unsigned long *) a[1], 4 * sizeof(unsigned long)))
						ret = audit_set_prog_cap((int) prog_a[0], (char *) prog_a[1], prog_a[2], (kdev_t) prog_a[3]);

					break;
				}
				default : {
					ret = -EINVAL;
				}
			}

			break;
		}
		case AU_CAP_GET : {
			int a[2], cap[] = {0, 0, 0};

			if (copy_from_user(&a, args, 2 * sizeof(unsigned long)))
				return -EFAULT;
	
			ret = audit_get_capability((int) a[0], (int *) &cap);

			if (!ret)
				if (copy_to_user((void *) a[1], &cap, 3 * sizeof(int)))
					ret = -EFAULT;

			break;
		}
		case AU_CAP_CHMOD : {
			ret = audit_setmod((int) *args);
			break;
		}
		case AU_CAP_GETADMIN : {
			int retval;

			retval = audit_getadmin();
			if (put_user((unsigned long) retval, args))
				return -EFAULT;
			
			break;
		}
		case AU_CAP_GETMOD : {
			ret = audit_getmodus();
			break;
		}
	}

	return ret;
}
