#include <asm/uaccess.h>

#include <linux/sched.h>
#include <linux/string.h>
#include <linux/utsname.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/mount.h>
#include <linux/mm.h>
#include <linux/dcache.h>
#include <linux/smp_lock.h>

#include <linux/audit.h>


#if defined CONFIG_AUDIT || defined CONFIG_ACL

int audit_fixpath(char *s, unsigned int ls)
{
	int id;
	int is;
	int slashseen;

	slashseen = 0;
	for (is = 0, id = 0; is < ls; is++) {
		if (s[is] == '\0') {
			if (id > 1 && s[id-1] == '/') {
				--id;
			}
			break;
		}
		if (slashseen) {
			if (s[is] == '/')
				continue;
		} else if (s[is] == '/') {
			slashseen = 1;
			s[id++] = '/';
			continue;
		}
		if (s[is] == '.' && s[is+1] == '/') {
			is += 1;
			continue;
		}
		if (s[is] == '.' && s[is+1] == '\0') {
			if (id > 1) id--;
			continue;
		}
		if (s[is] == '.' && s[is+1] == '.' && s[is+2] == '\0') {
			is += 1;
			if (id > 0) id--;
			while (id > 0 && s[--id] != '/');
			id++;
			continue;
		}
		if (s[is] == '.' && s[is+1] == '.' && s[is+2] == '/') {
			is += 2;
			if (id > 0) id--;
			while (id > 0 && s[--id] != '/');
			id++;
			continue;
		}
		while (is < ls && (s[id++] = s[is++]) != '/' && s[is] != '\0');
		is--;
	}

	return (id);
}


unsigned long audit_resolve_dentry(char *dest, struct dentry *d,
                                   struct vfsmount *mp)
{
	struct dentry *cwd = d;
	struct vfsmount *cmp = mp;

	unsigned long path_len = 0;
	unsigned short path_pos = 0;

	if (!cwd && ! cmp)
		return 0;

	for (;;) {
		if (IS_ROOT(cwd)) {
			if (cmp->mnt_parent == cmp) {
				if (!path_len) {
					*dest = '/';
					return 1;
				}

				break;
			}
			
			cwd = cmp->mnt_mountpoint;
			cmp = cmp->mnt_parent;
		}

		path_len += cwd->d_name.len;
		memcpy(dest + ((PATH_MAX + 1) - path_len), cwd->d_name.name,
		       cwd->d_name.len);

		*(dest + ((PATH_MAX + 1) - (++path_len))) = '/';

		if (!(cwd = cwd->d_parent)) {
#ifdef CONFIG_AUDIT_DEBUG
			printk(KERN_DEBUG "audit_resolve_dentry: zero pointer in cwd \n");
#endif
			return 0;
		}
	}

	path_pos = (unsigned short) (PATH_MAX + 1) - path_len;
	path_len = audit_fixpath(dest + path_pos, path_len);

	path_len += path_pos << 16;

	return path_len;
}
#endif


#ifdef CONFIG_AUDIT
void audit_resolve_filename(char *dest, unsigned long *len,
                            const char *filename, int f_size)
{
	char *tmp = NULL, *tmp_p = NULL;
	unsigned long path_chr, path_len = 0;
	struct dentry *cwd;
	struct vfsmount *vfsmnt;

	if (!len)
		return;

	*len = 0;

	if (!dest || !filename)
		return;

	if (!current->fs)
		return;

	cwd = current->fs->pwd;
	vfsmnt = current->fs->pwdmnt;

	if (!cwd || !vfsmnt) {
#ifdef CONFIG_AUDIT_DEBUG
		printk(KERN_NOTICE "audit_resolve_filename: "
		                   "error in the fs_struct of the current process.\n");
#endif
		return;
	}

	if ((unsigned long) filename < TASK_SIZE) {
		tmp = getname(filename);

		if (IS_ERR(tmp))
			return;

	} else {
		tmp = __getname();

		if (!tmp)
			return;

		memcpy(tmp, filename, PATH_MAX + 1);
	}

	tmp_p = tmp;

	if (*tmp_p == '/') {
		if (!current->fs->root) {
#ifdef CONFIG_AUDIT_DEBUG
			printk(KERN_DEBUG "audit_resolve_filename: "
			           "fs_struct of the current process has no root entry.\n");
#endif
			return;
		}

		if (IS_ROOT(current->fs->root)) {
			if (vfsmnt != vfsmnt->mnt_parent) {
				struct vfsmount *vfstmp = vfsmnt->mnt_parent;

				for (;;) {
					if (!vfstmp) {
#ifdef CONFIG_AUDIT_DEBUG
						printk(KERN_DEBUG "audit_resolve_filename: "
						           "error in vfsmnt of the current process.\n");
#endif
						return;
					}
						
					if (vfstmp == vfstmp->mnt_parent)
						break;

					vfstmp = vfstmp->mnt_parent;
				}

				if (vfstmp->mnt_root != current->fs->root)
					goto get_root;
			}
			path_len = f_size;

			memcpy(dest, tmp_p, path_len);

			path_len = audit_fixpath(dest, path_len);
			*len = path_len;

			goto put_out;
		}

get_root:
		cwd = current->fs->root;
		vfsmnt = current->fs->rootmnt;

		tmp_p++;
		f_size--;
	}

	memcpy(dest + ((PATH_MAX + 1) - f_size), tmp_p, f_size);
	path_len = f_size;

	*(dest + ((PATH_MAX + 1) - (++path_len))) = '/';

	if (cwd) {
		for (;;) {
			if (IS_ROOT(cwd)) {
				if (vfsmnt->mnt_parent == vfsmnt)
					break;

				cwd = vfsmnt->mnt_mountpoint;
				vfsmnt = vfsmnt->mnt_parent;
			}
				
			path_len += cwd->d_name.len;
			memcpy(dest + ((PATH_MAX + 1) - path_len), cwd->d_name.name,
			       cwd->d_name.len);

			*(dest + ((PATH_MAX + 1) - (++path_len))) = '/';

			cwd = cwd->d_parent;
			if (!cwd) {
#ifdef CONFIG_AUDIT_DEBUG
				printk(KERN_NOTICE "audit_resolve_filename: "
				                   "zero pointer in cwd \n");
#endif
				break;
			}
		}
	}

	path_chr = (PATH_MAX + 1) - path_len;
	path_len = audit_fixpath(dest + path_chr, path_len);

	path_len += path_chr << 16;
	*len = path_len;

put_out:
	putname(tmp);
	return;
}


inline void audit_fill_static(struct audit_host_record *rec)
{
	rec->ruid = current->uid;
	rec->rgid = current->gid;
	rec->euid = current->euid;
	rec->egid = current->egid;

	if (current->audit) {
		rec->aid = current->audit->aid;
		rec->session = current->audit->sid;
		rec->from = current->audit->from;
	} else {
		rec->aid = 0;
		rec->session = 0;
		rec->from = 0;
	}

	rec->tty = (current->tty) ? current->tty->device : 0;
}


inline void audit_fill_filedata(struct audit_host_record *rec, struct inode *i)
{
	if (i) {
		rec->inode = i->i_ino;

		if (S_ISCHR(i->i_mode) || S_ISBLK(i->i_mode))
			rec->dev = i->i_rdev;
		else
			rec->dev = i->i_dev;
	
		rec->owner = i->i_uid;
		rec->gowner = i->i_gid;
		rec->perm = (unsigned long) i->i_mode;

		return;
	} 

	rec->owner = 0;
	rec->gowner = 0;
	rec->dev = 0;
	rec->perm = 0;
	rec->inode = 0;
}


inline void audit_fill_ffiledata(struct audit_host_record *rec, struct file *f)
{
	if (f->f_dentry != NULL) {

		audit_fill_filedata(rec, f->f_dentry->d_inode);

		return;
	}

	audit_fill_filedata(rec, NULL);
}

char *audit_getname(const char *filename)
{
	char *name;

	name = __getname();
	if (name) {
		if ((unsigned long) filename < TASK_SIZE) {
			int err, len;

			len = strlen_user(filename) - 1;
			if (len > (PATH_MAX - 1))
				len = PATH_MAX - 1;

			err = strncpy_from_user(name, filename, len);
			if (err != len) {
#ifdef CONFIG_AUDIT_DEBUG
				printk(KERN_DEBUG "audit_getname: "
				                  "strncpy_from_user() returned %d instead of %d.\n",
				       err, len);
#endif

				putname(name);
				return ERR_PTR(-EFAULT);;
			}

			name[len] = '\0';
		} else
			memcpy(name, filename, PATH_MAX);
	} else
		name = ERR_PTR(-ENOMEM);

	return name;
}


int audit_fill_cfiledata(struct audit_host_record *rec, const char *filename)
{
	int err = 0;

	if (filename) {
		char *name = NULL;

		name = audit_getname(filename);
		if (!IS_ERR(name)) {
			struct nameidata nd;

			if (path_init(name, LOOKUP_FOLLOW|LOOKUP_POSITIVE, &nd))
				err = path_walk(name, &nd);

			if (!err) {
				struct inode *d_inode;

				d_inode = nd.dentry->d_inode;

#ifdef CONFIG_AUDIT_DEBUG
				if (!d_inode) {
					printk(KERN_DEBUG "audit_fill_cfiledata: "
					                  "no inode given for %s\n", filename);
				}
#endif

				audit_fill_filedata(rec, d_inode);
				path_release(&nd);

				putname(name);
				return 0;
			}

			putname(name);
		} else {
			err = PTR_ERR(name);
#ifdef CONFIG_AUDIT_DEBUG
			printk(KERN_DEBUG "audit_fill_cfiledata: can't get the name %d.\n",
			       err);
#endif
		}
	}

	audit_fill_filedata(rec, NULL);

	return err;
}
#endif
