#ifndef __AUDIT_PROC_H
#define __AUDIT_PROC_H

#include <linux/config.h>
#include <linux/spinlock.h>
#include <linux/audit.h>

#ifdef CONFIG_AUDIT_DEBUG

struct au_proc_buf_index_s {
	unsigned int to_save_index;
	unsigned int to_save_state;
	void *to_save_addr;

	unsigned int first_full_index;
	void *first_full_addr;

	unsigned int first_free_index;
	void *first_free_addr;

	unsigned int additional;
};

#define AU_PROC_BUF_INDEX_INIT \
{ 							\
	to_save_index: 0, \
	to_save_addr: NULL, \
	first_free_index: 0, \
	first_free_addr: NULL, \
	first_full_index: 0, \
	first_full_addr: NULL, \
	additional: 0 \
}

struct au_proc_rec_s {
	unsigned long type;
	time_t        date;
	pid_t         pid;
	int           event;
	unsigned int  state;
};

struct au_svc_proc_s {
	struct au_proc_buf_index_s buf_start;
	struct au_proc_buf_index_s buf_end;
	
	struct au_proc_rec_s last_rec;

	int error;
	unsigned int err_getdata:1,
	             err_buf2file:1,
	             err_rec2chr:1,
	             pad:29;

	unsigned int rec2chr;
	unsigned int rec2chr_full;
	unsigned int rec2chr_free;
	unsigned int rec2chr_unused;
	unsigned int rec2chr_used;
	unsigned int rec2chr_err;

	unsigned int buf2file;

	unsigned int nxt2save;

	unsigned int enodata;
	unsigned int getdata;

	int retval;
	int size;
	unsigned long saved;
	unsigned long limit;

	spinlock_t lock;
};

#define AU_SVC_PROC_INIT \
{ \
	buf_start: AU_PROC_BUF_INDEX_INIT, \
	buf_end: AU_PROC_BUF_INDEX_INIT, \
	lock: SPIN_LOCK_UNLOCKED \
}

static inline void audit_proc_save_last_rec(struct au_proc_rec_s *proc_rec,
                                            struct audit_record *rec)
{
	if (rec)
		memcpy((char *) proc_rec, (char *) rec, sizeof(struct au_proc_rec_s));
}

static inline void audit_proc_buf_ptr(struct au_proc_buf_index_s *buf,
                                      struct audit_buf *svc)
{
	buf->to_save_index = svc->to_save->index;
	buf->to_save_state = svc->to_save->state;
	buf->to_save_addr = (void *) svc->to_save;

	buf->first_free_index = svc->first_free->index;
	buf->first_free_addr = (void *) svc->first_free;
	if (svc->first_full) {
		buf->first_full_index = svc->first_full->index;
		buf->first_full_addr = (void *) svc->first_full;
	}

	if (!svc->to_save->prev || svc->to_save->prev == svc->to_save)
		buf->additional = 1;
}

extern struct au_svc_proc_s au_svc_proc;

#define AUDIT_DEBUG_PROC_VAR_INC(var) {au_svc_proc.var++;}
#define AUDIT_DEBUG_PROC_VAR_SET(var, val) {au_svc_proc.var = val;}

#define AUDIT_DEBUG_PROC_ERR(func, err) \
{ \
	if (err < 0) { \
		au_svc_proc.error = err; \
		au_svc_proc.err_##func = 1; \
	} \
}

#else 

#define AUDIT_DEBUG_PROC_VAR_INC(var) {do { } while (0);}
#define AUDIT_DEBUG_PROC_VAR_SET(var, val) {do { } while (0);}
#define AUDIT_DEBUG_PROC_ERR(func, err) {do { } while (0);}

#endif

extern void audit_proc_init(void);

#endif

