#include <asm/uaccess.h>

#include <linux/errno.h>

#include <linux/acl.h>

#include <linux/audit.h>
#include <linux/audit_event.h>

extern int audit_getmodus(void);

asmlinkage long sys_auditcall(int call, unsigned long *args)
{
	int err;

	if (call < SYS_AUDIT_SERVICE || call > SYS_GET_MAGIC)
		return -EINVAL;

	err = 0;
	switch (call) {
#ifdef CONFIG_AUDIT
		case SYS_AUDIT_SERVICE : {
			unsigned long a[2];

			if (copy_from_user(a, args, 2 * sizeof(long)))
				return -EFAULT;

			err = sys_audit_service((int) a[0], a[1]);

			if (audit_getmodus() == AU_DISABLED) {
				sys_audit_ctl(AU_OFF, 0);

				err = -ESRCH;
			}

			break;
		}
		case SYS_AUDIT_CTL: {
			unsigned long a[2];

			if (copy_from_user(a, args, 2 * sizeof(long)))
				return -EFAULT;

			err = sys_audit_ctl(a[0], a[1]);
			break;
		}
		case SYS_AUDIT_EVENT: {
			unsigned long a[3];

			if (audit_getmodus() == AU_DISABLED)
				return -ESRCH;

			if (copy_from_user(a, args, 3 * sizeof(long)))
				return -EFAULT;

			err = sys_audit_event((int) a[0], (int) a[1],
			                      (struct audit_app_event *) a[2]);

			break;
		}
#endif
#ifdef CONFIG_ACL
		case SYS_ACL_CTL: {
			unsigned long a[3];

			if (copy_from_user(a, args, 3 * sizeof(long)))
				return -EFAULT;

			err = sys_acl_ctl((int) a[0], (unsigned long) a[1],
			                  (unsigned int) a[2]);
			break;
		}
#endif
		case SYS_GET_MAGIC:
			err = AU_MAGIC_BASIC;
#ifdef CONFIG_AUDIT
			err |= AU_MAGIC_AUDIT;
#endif
#ifdef CONFIG_ACL
			err |= AU_MAGIC_ACL;
#endif
			break;
		default:
			err = -ENOSYS;
	}

	return err;
}
