#ifndef _AUDIT_H
#define _AUDIT_H

#include <asm/param.h>

#include <linux/config.h>
#include <linux/linkage.h>
#include <linux/limits.h>
#include <linux/if_ether.h>

#include <linux/acl.h>

#ifdef __KERNEL__

#  include <asm/ptrace.h>
#  include <asm/byteorder.h>

#  include <linux/sched.h>
#  include <linux/types.h>
#  include <linux/kdev_t.h>
#  include <linux/fs.h>
#  include <linux/utime.h>
#  include <linux/spinlock.h>
#  include <linux/netdevice.h>

#  include <linux/skbuff.h>
#  include <linux/if.h>
#  include <linux/if_arp.h>
#  include <linux/in.h>
#  include <linux/ip.h>
#  include <linux/igmp.h>
#  include <linux/icmp.h>
#  include <linux/udp.h>
#  include <linux/tcp.h>

#  if defined(CONFIG_IPV6) || defined (CONFIG_IPV6_MODULE)
#    include <linux/ipv6.h>
#    include <linux/icmpv6.h>
#  endif

#else

#  include <sys/types.h>
#  include <asm/types.h>

#  define AU_MASK_SIZE 	8
#  define AU_INIT_MASK {0, 0, 0, 0, 0, 0, 0, 0}

typedef dev_t kdev_t;

#endif 


#define STRLEN(x) ((x) ? strlen(x) : 0)
#define STRLEN_USER(x) ((x) ? strlen_user(x) : 0)

#define AU_DISABLED		200
#define AU_NORMAL		201
#define AU_SECURE		202

#define AU_CAP_ADM		1
#define AU_CAP_SETID	2
#define AU_CAP_MSG		3

#define AU_CAP_SET		10
#define AU_CAP_GET		11
#define AU_CAP_CHMOD	12
#define AU_CAP_GETADMIN	13
#define AU_CAP_GETMOD	14

#define AU_CAP_SET_AID	20
#define AU_CAP_SET_PROG	21

#define AU_UNDEF 		-1
#define AU_OK 			0
#define AU_DOIT 		30
#define AU_STOPPED 		31

#define AU_FREE			100
#define AU_USED			101
#define AU_FULL			102
#define AU_UNUSED		103
#define AU_SEND_ALRM	104

#define AU_SYSCALL_FAILED 		-1
#define AU_SYSCALL_SUCCESSFULL	0

#define AU_BUFFERMIN 			50
#define AU_MIN_WATERMARK		5
#define AU_MAX_TRAILSIZE 		524288000 

#define AU_FS_HASH 77
#define AU_FS_HASH_FUNC(x, c, h) ((64 * x + (int) c) % h)

#define SYS_AUDIT_SERVICE	1
#define SYS_AUDIT_CTL		2
#define SYS_AUDIT_EVENT		3
#define SYS_ACL_CTL			4
#define SYS_GET_MAGIC		5

#define AU_MAGIC_BASIC 	0x00040000
#define AU_MAGIC_AUDIT 	0x00000001
#define AU_MAGIC_ACL 	0x00000002

#define AU_ON 			10
#define AU_OFF 			11
#define AU_SETAID		12
#define AU_LOGIN		13
#define AU_INITMASK		14
#define AU_SETMASK		15
#define AU_BUFFER_CTL	16
#define AU_CAP_CTL		17
#define AU_GETMASK		18

#define AU_SETMASK_ALL	0
#define AU_SETMASK_PID	1
#define AU_SETMASK_UID	2
#define AU_SETMASK_GID	3

#define AU_GETMASK_PID	30
#define AU_GETMASK_UID	31
#define AU_GETMASK_GID	32

#define AU_BUF_INIT				40
#define AU_BUF_DESTROY			41
#define AU_BUF_CONFIG			42
#define AU_BUF_SET_SIZE			43
#define AU_BUF_SET_WM			44
#define AU_DYN_BUF_SET_WM		45
#define AU_DYN_BUF_SET_ORDER	46


#define AU_BUFMIN		50

#define AU_REC_TYPES	 2
#define AU_REC_TYPES_INIT {NULL, NULL}

#define AU_HOST_REC		 0
#define AU_NET_REC 		 1

struct audit_app_event {
	int				status;

	unsigned long	arg1;
	unsigned long	arg2;

	char			file[PATH_MAX + 1];
	char			file2[PATH_MAX + 1];

	char			**env;
};

#define AU_EVENT_INIT {0, 0, 0, "", "", NULL}

struct audit_prog_id {
	char 			comm[16];
	unsigned long 	inode;
	kdev_t			dev;
};

struct audit_cap_db {
	int aid;
	struct audit_prog_id prog_id;

	int cap;
	struct audit_cap_db *next;
};

#define AU_MAX_DYN_BUF_ENTRIES	24
#define AU_MAX_DYN_BUF_SIZE	\
	(sizeof(unsigned long) * (AU_MAX_DYN_BUF_ENTRIES + 1) + 131072)

#define AU_WRITE_BUFFER_SIZE	(8352 + AU_MAX_DYN_BUF_SIZE)

#define AU_NET_MAC_RAW_SIZE 16
#define AU_NET_NH_RAW_SIZE  40
#define AU_NET_H_RAW_SIZE   20

#ifdef __KERNEL__


struct audit_dyn_buf {
	unsigned int page_size;
	unsigned int page_order;

	int state;
	int wm;

	int size;

	spinlock_t lock_buf;

	struct audit_dyn_buf_entry *data;
};


struct audit_dyn_buf_entry {
	int pages;
	int state;

	int entries;

	char *addr;

	char *entry[AU_MAX_DYN_BUF_ENTRIES];
	unsigned int entry_len[AU_MAX_DYN_BUF_ENTRIES];

	struct audit_dyn_buf_entry *next;
	struct audit_dyn_buf *head;
};


struct audit_buffer_info {
	unsigned int state;
	unsigned int type;

	unsigned int rec_size;
	unsigned int buf_size;
	unsigned int buf_wm;
	unsigned int free_recs;

	unsigned long alrm_cnt;

	unsigned long dropped;
	unsigned long filtered;

	void *buf_addr;
	void *cur_rec;

	int (*init) (struct audit_buffer_info *, unsigned int);
	int (*destroy) (struct audit_buffer_info *);
	int (*reduse) (struct audit_buffer_info *);

	int (*ctl) (struct audit_buffer_info *, unsigned long, unsigned long);

	void * (*get_free) (struct audit_buffer_info *);
	void * (*get_new) (void);
	int (*rec2chr) (struct audit_buffer_info *, char *, void *);
	int (*free_rec) (struct audit_buffer_info *, void *);

	int (*buf_suspend) (void);
	void (*buf_wake_up) (void);
};

#define AU_BUFFER_INFO { \
	state: AU_UNDEF, \
	type: AU_UNDEF, \
	rec_size: 0,\
	buf_size: 0,\
	buf_wm: 0,\
	free_recs: 0,\
	alrm_cnt: 0,\
	dropped: 0,\
	filtered: 0,\
	buf_addr: NULL,\
	cur_rec: NULL,\
	init: NULL,\
	destroy: NULL,\
	reduse: NULL,\
	ctl: NULL,\
	get_free: NULL,\
	get_new: NULL,\
	rec2chr: NULL,\
	free_rec: NULL,\
	buf_suspend: NULL,\
	buf_wake_up: NULL \
}

struct audit_host_record {

	int           aid;  	
	uid_t         ruid; 	
	gid_t         rgid; 	
	uid_t         euid; 	
	gid_t         egid; 	
	pid_t         session;	
	unsigned long tty;	    
	int           status;	
	uid_t         owner;	
	gid_t         gowner;	
	unsigned long dev;		
	unsigned long perm;		
	unsigned long inode;	
	unsigned long from;		
	unsigned long arg1;		
	unsigned long arg2;		

	struct audit_dyn_buf_entry *env;

	char          file[PATH_MAX + 1];
	char          file2[PATH_MAX + 1];
	unsigned long file_len;
	unsigned long file2_len;

	struct audit_host_record *next;
	unsigned int state;
};

#define AU_HOST_REC_INIT { \
	0, 0, 0, 0, \
	0, 0, 0, \
	0, 0, \
	0, 0, 0, 0, 0, \
	0, 0, 0, \
	NULL, \
	"", "", \
	0, 0, \
	NULL, AU_FREE \
}

#ifdef CONFIG_NET_AUDIT

struct audit_net_record {
	unsigned short protocol;
	unsigned short dev_type;
	
	unsigned char  mac_prot_size;
	unsigned char  nh_prot_size;
	unsigned char  h_prot_size;

	unsigned char  namelen;

	union {
	  	struct ethhdr ethernet;
		unsigned char raw[AU_NET_MAC_RAW_SIZE];
	} mac;

#define eth_dest mac.ethernet.h_dest
#define eth_source mac.ethernet.h_source
#define eth_proto mac.ethernet.h_proto

	union {
		struct iphdr      iph;
		struct arphdr     arph;
#if defined(CONFIG_IPV6) || defined (CONFIG_IPV6_MODULE)
		struct ipv6hdr    ipv6h;
#endif
		unsigned char     raw[AU_NET_NH_RAW_SIZE];
	} nh;
		
#define au_iph iph.iph

	union {
		struct icmphdr    icmph;
		struct tcphdr     tcph;
		struct udphdr     udph;
		struct iphdr      ipiph;
		struct igmphdr    igmph;
#if defined(CONFIG_IPV6) || defined (CONFIG_IPV6_MODULE)
		struct icmp6hdr   icmp6h;
#endif
		unsigned char     raw[AU_NET_H_RAW_SIZE];
	} h;

	char           name[IFNAMSIZ];

	struct audit_dyn_buf_entry *env;

	struct audit_net_record *next;
	unsigned int state;
};

#define AU_NET_REC_INIT { \
	"", "", 0, \
	0, {0, 0, 0, 0}, {0, 0, 0, 0}, 0, \
	0, 0, 0, 0, \
	0, \
	"", "", \
	0, \
	NULL, AU_FREE \
}

#endif 

struct audit_record {
	unsigned long type;
	time_t        date;
	pid_t         pid;
	int           event;

	unsigned int  state;

	unsigned long hostname_len;
	char          hostname[MAXHOSTNAMELEN + 4];

	void   *data;

	unsigned int index;

	struct audit_record *next;
	struct audit_record *prev;
};

#endif

#define AU_BUF_FIX      16
#define AU_HOST_REC_FIX	64
#define AU_NET_REC_FIX  8

#ifdef __KERNEL__


extern int sys_audit_event(int type, int id, void *data);
extern int sys_audit_ctl(int cmd, unsigned long args);
extern int sys_audit_service(int, unsigned long);

#define AUDIT_GET_FAM(tp) ((!tp->af_specific) ? 0 : \
	((!tp->af_specific->net_header_len == sizeof(struct iphdr)) ? AF_INET : \
	((!tp->af_specific->net_header_len == sizeof(struct ipv6hdr) ? AF_INET6 : 0))))

extern pid_t audit_get_pid(void);

extern int wake_up_auditd(void);
extern int audit_drop_evt(unsigned int type, int cnt);

extern int audit_doit(unsigned int event, int status);

#ifdef CONFIG_NET_AUDIT

extern int audit_net_doit(unsigned int evt, unsigned int subevt,
                          struct sk_buff *skb, int fam);
extern int audit_init_net(struct audit_buffer_info **info);
extern int audit_check_net_buffer(void);

extern int audit_no_proto(struct sk_buff *skb, unsigned int sub_evt,
                          struct net_device *dev);

#if defined(CONFIG_IPV6) || defined (CONFIG_IPV6_MODULE)
extern int audit_ipv6_rcv(struct sk_buff *skb, unsigned short sub_evt,
                          short exthdr);
#endif
extern int audit_arp_rcv(struct sk_buff *skb, unsigned int sub_evt);
extern int audit_ipv4_rcv(struct sk_buff *skb, unsigned int sub_evt);


#if defined(CONFIG_IPV6) || defined (CONFIG_IPV6_MODULE)
extern int audit_icmp_v6(struct sk_buff *skb, unsigned int sub_evt);
#endif

extern int audit_icmp_v4(struct sk_buff *skb, unsigned int sub_evt);
extern int audit_tcp_rcv(struct sk_buff *skb, unsigned int sub_evt, 
                         struct sock *sk);
extern int audit_tcp_to(struct sock *sk, unsigned int sub_evt);
extern int audit_udp_rcv(struct sk_buff *skb, unsigned int sub_evt);
extern int audit_ipip_rcv(struct sk_buff *skb, unsigned int sub_evt);
extern int audit_igmp_rcv(struct sk_buff *skb, unsigned int sub_evt);

extern int audit_copy_dev(struct audit_net_record *data, struct net_device *dev,
                          unsigned int len);
extern int audit_copy_mac(struct audit_net_record *data, struct sk_buff *skb,
                          struct net_device *dev);
extern int audit_copy_ipv4(struct audit_net_record *data, struct iphdr *iphr);

#if defined(CONFIG_IPV6) || defined (CONFIG_IPV6_MODULE)
extern int audit_copy_ipv6(struct audit_net_record *data, struct ipv6hdr *iphr,
                           __u8 nexthdr);
#endif

#endif 

extern int audit_buffer_ctl(int cmd, unsigned long *args);
extern int audit_buffer_rollback(void);

extern int audit_check_capability(int);
extern int sys_audit_cap_ctl(int, unsigned long *);
extern int audit_setadmin(uid_t admin);
extern int audit_setmod(int mod);

extern void audit_resolve_filename(char *, unsigned long *, const char *, int);
extern unsigned long audit_resolve_dentry(char *, struct dentry *,
                                          struct vfsmount *);
extern inline void audit_fill_static(struct audit_host_record *);
extern inline void audit_fill_filedata(struct audit_host_record *,
                                       struct inode *);
extern inline void audit_fill_ffiledata(struct audit_host_record *,
                                        struct file *);
extern int audit_fill_cfiledata(struct audit_host_record *, const char *);


extern void audit_evt_filtered(unsigned int type);


extern struct audit_record *audit_get_free_record(unsigned int type,
                                                  int interruptable,
                                                  unsigned int evt);

extern int audit_init_host(struct audit_buffer_info **info);


static inline void audit_buffer_init(struct audit_buffer_info** infos)
{
	audit_init_host(infos);
#ifdef CONFIG_NET_AUDIT
	audit_init_net(infos);
#endif
}


#define AU_K_SPACE 0
#define AU_U_SPACE 1

extern struct audit_dyn_buf *audit_dyn_buf_init(char *name);
extern int audit_fill_dyn_buf(struct audit_dyn_buf *head,
                              struct audit_dyn_buf_entry **entry,
                              void *src[], unsigned int src_entries);
extern int audit_fill_dyn_buf2(struct audit_dyn_buf *head,
                               struct audit_dyn_buf_entry **dest, void *src[],
                               unsigned int entries, int from);
extern int audit_dyn_buf_update_wm(struct audit_dyn_buf *head, int wm);
extern int audit_dyn_update_order(struct audit_dyn_buf **head, int order);
extern int audit_dyn_rec2chr(char *buf, struct audit_dyn_buf_entry *entry);
extern int audit_resize_dyn_buf(struct audit_dyn_buf *head);
extern int audit_delete_dyn_buf(struct audit_dyn_buf *buf);

extern void *orig_sys_call_table[];

extern int audit_sysboot(void);
extern int audit_setaid(int, int, int);

extern int audit_vfork(int retval);

extern int audit_ioperm(int, unsigned long, unsigned long, int);
extern int audit_iopl(int, unsigned long);

extern int audit_quotactl(unsigned long *);

extern int audit_link(unsigned long *);
extern int audit_unlink(int, const char *, struct inode *);
extern int audit_mknod(unsigned long *);
extern int audit_rename(unsigned long *);
extern int audit_mkdir(unsigned long *);
extern int audit_rmdir(int, const char *, struct inode *);
extern int audit_symlink(unsigned long *);

extern int audit_open(unsigned long *);
extern int audit_chdir(int, const char *);
extern int audit_chmod(unsigned long *);
extern int audit_utime(int, const char *, struct utimbuf *);
extern int audit_utimes(int, const char *, struct timeval *);
extern struct audit_record *audit_chroot_pre(const char *);
extern int audit_chroot_post(struct audit_record *, int);
extern int audit_truncate(int, const char *, unsigned long);
extern int audit_ftruncate(int, unsigned int, unsigned long);
extern int audit_fchmod(unsigned long *);
extern int audit_fchdir(int, unsigned int);
extern int audit_mmap_exec(int status, unsigned long fd, unsigned long prot,
                           unsigned long flags);

extern int audit_mount(unsigned long *);

extern int audit_acct(int, const char *);

extern int audit_capset(int);

extern int audit_delete_module(int, const char *);
extern int audit_init_module(int, const char *, struct module *);

extern int audit_syslog(int, int);

extern int audit_stime(int, int *);
extern int audit_settimeofday(int, struct timeval *);
extern int audit_adjtimex(int);

extern int audit_swapon(int, const char *, int);
extern int audit_swapoff(int, const char *);

#endif

#endif
